@@ -1655,7 +1655,7 @@ int ctdb_start_daemon(struct ctdb_context *ctdb,
 
 	initialise_node_flags(ctdb);
 
-	ret = ctdb_set_public_addresses(ctdb, true);
+	ret = ctdb_set_public_addresses(ctdb);
 	if (ret == -1) {
 		D_ERR("Unable to setup public IP addresses\n");
 		exit(1);
@@ -2197,7 +2197,7 @@ int ctdb_control_getnodesfile(struct ctdb_context *ctdb,
 
 	CHECK_CONTROL_DATA_SIZE(0);
 
-	node_map = ctdb_read_nodes(ctdb, ctdb->nodes_file);
+	node_map = ctdb_read_nodes(ctdb, ctdb->nodes_source);
 	if (node_map == NULL) {
 		D_ERR("Failed to read nodes file\n");
 		return -1;
