@@ -91,6 +91,18 @@ static const struct enum_list enum_bool_auto[] = {
 	{-1, NULL}
 };
 
+static const struct enum_list enum_bool_auto_default[] = {
+	{false, "No"},
+	{false, "False"},
+	{false, "0"},
+	{true, "Yes"},
+	{true, "True"},
+	{true, "1"},
+	{Auto, "Auto"},
+	{LP_ENUM_Default, "default"},
+	{-1, NULL}
+};
+
 static const struct enum_list enum_csc_policy[] = {
 	{CSC_POLICY_MANUAL, "manual"},
 	{CSC_POLICY_DOCUMENTS, "documents"},
@@ -449,6 +461,14 @@ static const struct enum_list enum_acl_claims_evaluation[] = {
 	{-1, NULL}
 };
 
+static const struct enum_list enum_client_netlogon_ping_protocol[] = {
+	{CLIENT_NETLOGON_PING_CLDAP, "cldap"},
+	{CLIENT_NETLOGON_PING_LDAP, "ldap"},
+	{CLIENT_NETLOGON_PING_LDAPS, "ldaps"},
+	{CLIENT_NETLOGON_PING_STARTTLS, "starttls"},
+	{-1, NULL}
+};
+
 /* Note: We do not initialise the defaults union - it is not allowed in ANSI C
  *
  * NOTE: Handling of duplicated (synonym) parameters:
