@@ -23,6 +23,8 @@
 #include "libcli/smb/smb_common.h"
 #include "system/filesys.h"
 #include "lib/param/loadparm.h"
+#include "lib/param/param.h"
+#include "libcli/smb/smb2_negotiate_context.h"
 
 const char *smb_protocol_types_string(enum protocol_types protocol)
 {
@@ -45,16 +47,10 @@ const char *smb_protocol_types_string(enum protocol_types protocol)
 		return "SMB2_02";
 	case PROTOCOL_SMB2_10:
 		return "SMB2_10";
-	case PROTOCOL_SMB2_22:
-		return "SMB2_22";
-	case PROTOCOL_SMB2_24:
-		return "SMB2_24";
 	case PROTOCOL_SMB3_00:
 		return "SMB3_00";
 	case PROTOCOL_SMB3_02:
 		return "SMB3_02";
-	case PROTOCOL_SMB3_10:
-		return "SMB3_10";
 	case PROTOCOL_SMB3_11:
 		return "SMB3_11";
 	}
@@ -68,7 +64,7 @@ const char *smb_protocol_types_string(enum protocol_types protocol)
 char *attrib_string(TALLOC_CTX *mem_ctx, uint32_t attrib)
 {
 	size_t i, len;
-	const struct {
+	static const struct {
 		char c;
 		uint16_t attr;
 	} attr_strs[] = {
@@ -166,40 +162,50 @@ mode_t wire_perms_to_unix(uint32_t perms)
         return ret;
 }
 
-/****************************************************************************
- Return the file type from the wire filetype for UNIX extensions.
-****************************************************************************/
 
-mode_t unix_filetype_from_wire(uint32_t wire_type)
+/****************************************************************************
+ * Return the file type from the wire filetype for UNIX extensions.
+ *
+ * This uses the fact that the unix file types are numbered from
+ * FILE=0 to SOCKET=6. This is an accepted protocol element that will
+ * never change.
+ ****************************************************************************/
+
+static const mode_t unix_filetypes[] =
+	{S_IFREG, S_IFDIR, S_IFLNK, S_IFCHR, S_IFBLK, S_IFIFO, S_IFSOCK};
+
+mode_t wire_filetype_to_unix(uint32_t wire_type)
 {
-	switch (wire_type) {
-		case UNIX_TYPE_FILE:
-			return S_IFREG;
-		case UNIX_TYPE_DIR:
-			return S_IFDIR;
-#ifdef S_IFLNK
-		case UNIX_TYPE_SYMLINK:
-			return S_IFLNK;
-#endif
-#ifdef S_IFCHR
-		case UNIX_TYPE_CHARDEV:
-			return S_IFCHR;
-#endif
-#ifdef S_IFBLK
-		case UNIX_TYPE_BLKDEV:
-			return S_IFBLK;
-#endif
-#ifdef S_IFIFO
-		case UNIX_TYPE_FIFO:
-			return S_IFIFO;
-#endif
-#ifdef S_IFSOCK
-		case UNIX_TYPE_SOCKET:
-			return S_IFSOCK;
-#endif
-		default:
-			return (mode_t)0;
+	if (wire_type >= ARRAY_SIZE(unix_filetypes)) {
+		return (mode_t)0;
 	}
+	return unix_filetypes[wire_type];
+}
+
+uint32_t unix_filetype_to_wire(mode_t mode)
+{
+	mode_t type = mode & S_IFMT;
+	size_t i;
+
+	for (i = 0; i < ARRAY_SIZE(unix_filetypes); i++) {
+		if (type == unix_filetypes[i]) {
+			return i;
+		}
+	}
+	return UNIX_TYPE_UNKNOWN;
+}
+
+mode_t wire_mode_to_unix(uint32_t wire)
+{
+	uint32_t wire_type = (wire & UNIX_FILETYPE_MASK) >>
+			     UNIX_FILETYPE_SHIFT;
+	return wire_perms_to_unix(wire) | wire_filetype_to_unix(wire_type);
+}
+
+uint32_t unix_mode_to_wire(mode_t mode)
+{
+	uint32_t wire_type = unix_filetype_to_wire(mode);
+	return unix_perms_to_wire(mode) | (wire_type << UNIX_FILETYPE_SHIFT);
 }
 
 bool smb_buffer_oob(uint32_t bufsize, uint32_t offset, uint32_t length)
@@ -216,7 +222,7 @@ bool smb_buffer_oob(uint32_t bufsize, uint32_t offset, uint32_t length)
 }
 
 /***********************************************************
- Common function for pushing stings, used by smb_bytes_push_str()
+ Common function for pushing strings, used by smb_bytes_push_str()
  and trans_bytes_push_str(). Only difference is the align_odd
  parameter setting.
 ***********************************************************/
@@ -396,7 +402,7 @@ static NTSTATUS internal_bytes_pull_str(TALLOC_CTX *mem_ctx, char **_str,
 	buf += offset;
 
 	if (ucs2) {
-		buf_len = utf16_len_n(buf, buf_len);
+		buf_len = utf16_null_terminated_len_n(buf, buf_len);
 	} else {
 		size_t tmp = strnlen((const char *)buf, buf_len);
 		if (tmp < buf_len) {
@@ -435,7 +441,7 @@ NTSTATUS smb_bytes_pull_str(TALLOC_CTX *mem_ctx, char **_str, bool ucs2,
  *
  * @param[in]  str  The string to translate.
  *
- * @return A corresponding enum @smb_signing_setting tranlated from the string.
+ * @return A corresponding enum @smb_signing_setting translated from the string.
  */
 enum smb_signing_setting smb_signing_setting_translate(const char *str)
 {
@@ -454,7 +460,7 @@ enum smb_signing_setting smb_signing_setting_translate(const char *str)
  *
  * @param[in]  str  The string to translate.
  *
- * @return A corresponding enum @smb_encryption_setting tranlated from the
+ * @return A corresponding enum @smb_encryption_setting translated from the
  *         string.
  */
 enum smb_encryption_setting smb_encryption_setting_translate(const char *str)
@@ -468,3 +474,308 @@ enum smb_encryption_setting smb_encryption_setting_translate(const char *str)
 
 	return encryption_state;
 }
+
+static const struct enum_list enum_smb3_signing_algorithms[] = {
+	{SMB2_SIGNING_AES128_GMAC, "AES-128-GMAC"},
+	{SMB2_SIGNING_AES128_CMAC, "AES-128-CMAC"},
+	{SMB2_SIGNING_HMAC_SHA256, "HMAC-SHA256"},
+	{-1, NULL}
+};
+
+const char *smb3_signing_algorithm_name(uint16_t algo)
+{
+	size_t i;
+
+	for (i = 0; i < ARRAY_SIZE(enum_smb3_signing_algorithms); i++) {
+		if (enum_smb3_signing_algorithms[i].value != algo) {
+			continue;
+		}
+
+		return enum_smb3_signing_algorithms[i].name;
+	}
+
+	return NULL;
+}
+
+static const struct enum_list enum_smb3_encryption_algorithms[] = {
+	{SMB2_ENCRYPTION_AES128_GCM, "AES-128-GCM"},
+	{SMB2_ENCRYPTION_AES128_CCM, "AES-128-CCM"},
+	{SMB2_ENCRYPTION_AES256_GCM, "AES-256-GCM"},
+	{SMB2_ENCRYPTION_AES256_CCM, "AES-256-CCM"},
+	{-1, NULL}
+};
+
+const char *smb3_encryption_algorithm_name(uint16_t algo)
+{
+	size_t i;
+
+	for (i = 0; i < ARRAY_SIZE(enum_smb3_encryption_algorithms); i++) {
+		if (enum_smb3_encryption_algorithms[i].value != algo) {
+			continue;
+		}
+
+		return enum_smb3_encryption_algorithms[i].name;
+	}
+
+	return NULL;
+}
+
+static int32_t parse_enum_val(const struct enum_list *e,
+			      const char *param_name,
+			      const char *param_value)
+{
+	struct parm_struct parm = {
+		.label = param_name,
+		.type = P_LIST,
+		.p_class = P_GLOBAL,
+		.enum_list = e,
+	};
+	int32_t ret = INT32_MIN;
+	bool ok;
+
+	ok = lp_set_enum_parm(&parm, param_value, &ret);
+	if (!ok) {
+		return INT32_MIN;
+	}
+
+	return ret;
+}
+
+struct smb311_capabilities smb311_capabilities_parse(const char *role,
+				const char * const *signing_algos,
+				const char * const *encryption_algos)
+{
+	struct smb311_capabilities c = {
+		.signing = {
+			.num_algos = 0,
+		},
+		.encryption = {
+			.num_algos = 0,
+		},
+	};
+	char sign_param[64] = { 0, };
+	char enc_param[64] = { 0, };
+	size_t ai;
+
+	snprintf(sign_param, sizeof(sign_param),
+		 "%s smb3 signing algorithms", role);
+	snprintf(enc_param, sizeof(enc_param),
+		 "%s smb3 encryption algorithms", role);
+
+	for (ai = 0; signing_algos != NULL && signing_algos[ai] != NULL; ai++) {
+		const char *algoname = signing_algos[ai];
+		int32_t v32;
+		uint16_t algo;
+		size_t di;
+		bool ignore = false;
+
+		if (c.signing.num_algos >= SMB3_ENCRYTION_CAPABILITIES_MAX_ALGOS) {
+			DBG_ERR("WARNING: Ignoring trailing value '%s' for parameter '%s'\n",
+				  algoname, sign_param);
+			continue;
+		}
+
+		v32 = parse_enum_val(enum_smb3_signing_algorithms,
+				     sign_param, algoname);
+		if (v32 == INT32_MAX) {
+			continue;
+		}
+		algo = v32;
+
+		for (di = 0; di < c.signing.num_algos; di++) {
+			if (algo != c.signing.algos[di]) {
+				continue;
+			}
+
+			ignore = true;
+			break;
+		}
+
+		if (ignore) {
+			DBG_ERR("WARNING: Ignoring duplicate value '%s' for parameter '%s'\n",
+				  algoname, sign_param);
+			continue;
+		}
+
+		c.signing.algos[c.signing.num_algos] = algo;
+		c.signing.num_algos += 1;
+	}
+
+	for (ai = 0; encryption_algos != NULL && encryption_algos[ai] != NULL; ai++) {
+		const char *algoname = encryption_algos[ai];
+		int32_t v32;
+		uint16_t algo;
+		size_t di;
+		bool ignore = false;
+
+		if (c.encryption.num_algos >= SMB3_ENCRYTION_CAPABILITIES_MAX_ALGOS) {
+			DBG_ERR("WARNING: Ignoring trailing value '%s' for parameter '%s'\n",
+				  algoname, enc_param);
+			continue;
+		}
+
+		v32 = parse_enum_val(enum_smb3_encryption_algorithms,
+				     enc_param, algoname);
+		if (v32 == INT32_MAX) {
+			continue;
+		}
+		algo = v32;
+
+		for (di = 0; di < c.encryption.num_algos; di++) {
+			if (algo != c.encryption.algos[di]) {
+				continue;
+			}
+
+			ignore = true;
+			break;
+		}
+
+		if (ignore) {
+			DBG_ERR("WARNING: Ignoring duplicate value '%s' for parameter '%s'\n",
+				  algoname, enc_param);
+			continue;
+		}
+
+		c.encryption.algos[c.encryption.num_algos] = algo;
+		c.encryption.num_algos += 1;
+	}
+
+	return c;
+}
+
+NTSTATUS smb311_capabilities_check(const struct smb311_capabilities *c,
+				   const char *debug_prefix,
+				   int debug_lvl,
+				   NTSTATUS error_status,
+				   const char *role,
+				   enum protocol_types protocol,
+				   uint16_t sign_algo,
+				   uint16_t cipher_algo)
+{
+	const struct smb3_signing_capabilities *sign_algos =
+		&c->signing;
+	const struct smb3_encryption_capabilities *ciphers =
+		&c->encryption;
+	bool found_signing = false;
+	bool found_encryption = false;
+	size_t i;
+
+	for (i = 0; i < sign_algos->num_algos; i++) {
+		if (sign_algo == sign_algos->algos[i]) {
+			/*
+			 * We found a match
+			 */
+			found_signing = true;
+			break;
+		}
+	}
+
+	for (i = 0; i < ciphers->num_algos; i++) {
+		if (cipher_algo == SMB2_ENCRYPTION_NONE) {
+			/*
+			 * encryption not supported, we'll error out later
+			 */
+			found_encryption = true;
+			break;
+		}
+
+		if (cipher_algo == ciphers->algos[i]) {
+			/*
+			 * We found a match
+			 */
+			found_encryption = true;
+			break;
+		}
+	}
+
+	if (!found_signing) {
+		/*
+		 * We negotiated a signing algo we don't allow,
+		 * most likely for SMB < 3.1.1
+		 */
+		DEBUG(debug_lvl,("%s: "
+		      "SMB3 signing algorithm[%u][%s] on dialect[%s] "
+		      "not allowed by '%s smb3 signing algorithms' - %s.\n",
+		      debug_prefix,
+		      sign_algo,
+		      smb3_signing_algorithm_name(sign_algo),
+		      smb_protocol_types_string(protocol),
+		      role,
+		      nt_errstr(error_status)));
+		return error_status;
+	}
+
+	if (!found_encryption) {
+		/*
+		 * We negotiated a cipher we don't allow,
+		 * most likely for SMB 3.0 and 3.0.2
+		 */
+		DEBUG(debug_lvl,("%s: "
+		      "SMB3 encryption algorithm[%u][%s] on dialect[%s] "
+		      "not allowed by '%s smb3 encryption algorithms' - %s.\n",
+		      debug_prefix,
+		      cipher_algo,
+		      smb3_encryption_algorithm_name(cipher_algo),
+		      smb_protocol_types_string(protocol),
+		      role,
+		      nt_errstr(error_status)));
+		return error_status;
+	}
+
+	return NT_STATUS_OK;
+}
+
+struct smb_transports smb_transports_parse(const char *param_name,
+					   const char * const *transports)
+{
+	struct smb_transports ts = {
+		.num_transports = 0,
+	};
+	size_t ti;
+
+	for (ti = 0; transports != NULL && transports[ti] != NULL; ti++) {
+		struct smb_transport t = {
+			.type = SMB_TRANSPORT_TYPE_UNKNOWN,
+		};
+		bool ignore = false;
+		size_t ei;
+		bool ok = false;
+
+		if (ts.num_transports >= SMB_TRANSPORTS_MAX_TRANSPORTS) {
+			DBG_ERR("WARNING: Ignoring trailing value '%s' for parameter '%s'\n",
+				transports[ti], param_name);
+			continue;
+		}
+
+		ok = smb_transport_parse(transports[ti], &t);
+		if (!ok) {
+			DBG_ERR("WARNING: Ignoring invalid value '%s' for parameter '%s'\n",
+				transports[ti], param_name);
+			continue;
+		}
+
+		for (ei = 0; ei < ts.num_transports; ei++) {
+			if (t.type != ts.transports[ei].type) {
+				continue;
+			}
+
+			if (t.port != ts.transports[ei].port) {
+				continue;
+			}
+
+			ignore = true;
+			break;
+		}
+
+		if (ignore) {
+			DBG_ERR("WARNING: Ignoring duplicate value '%s' for parameter '%s'\n",
+				transports[ti], param_name);
+			continue;
+		}
+
+		ts.transports[ts.num_transports] = t;
+		ts.num_transports += 1;
+	}
+
+	return ts;
+}
