@@ -21,6 +21,8 @@
 #include "idmap_cache.h"
 #include "../libcli/security/security.h"
 #include "../librpc/gen_ndr/idmap.h"
+#include "lib/gencache.h"
+#include "lib/util/string_wrappers.h"
 
 /**
  * Find a sid2xid mapping
@@ -35,7 +37,7 @@
 bool idmap_cache_find_sid2unixid(const struct dom_sid *sid, struct unixid *id,
 				 bool *expired)
 {
-	fstring sidstr;
+	struct dom_sid_buf sidstr;
 	char *key;
 	char *value = NULL;
 	char *endptr;
@@ -44,7 +46,7 @@ bool idmap_cache_find_sid2unixid(const struct dom_sid *sid, struct unixid *id,
 	struct unixid tmp_id;
 
 	key = talloc_asprintf(talloc_tos(), "IDMAP/SID2XID/%s",
-			      sid_to_fstring(sidstr, sid));
+			      dom_sid_str_buf(sid, &sidstr));
 	if (key == NULL) {
 		return false;
 	}
@@ -194,7 +196,8 @@ struct idmap_cache_xid2sid_state {
 	bool ret;
 };
 
-static void idmap_cache_xid2sid_parser(time_t timeout, DATA_BLOB blob,
+static void idmap_cache_xid2sid_parser(const struct gencache_timeout *timeout,
+				       DATA_BLOB blob,
 				       void *private_data)
 {
 	struct idmap_cache_xid2sid_state *state =
@@ -221,60 +224,10 @@ static void idmap_cache_xid2sid_parser(time_t timeout, DATA_BLOB blob,
 		state->ret = string_to_sid(state->sid, value);
 	}
 	if (state->ret) {
-		*state->expired = (timeout <= time(NULL));
+		*state->expired = gencache_timeout_expired(timeout);
 	}
 }
 
-/**
- * Find a uid2sid mapping
- * @param[in] uid		the uid to map
- * @param[out] sid		where to put the result
- * @param[out] expired		is the cache entry expired?
- * @retval Was anything in the cache at all?
- *
- * If "is_null_sid(sid)", this was a negative mapping.
- */
-
-bool idmap_cache_find_uid2sid(uid_t uid, struct dom_sid *sid, bool *expired)
-{
-	fstring key;
-	struct idmap_cache_xid2sid_state state;
-
-	fstr_sprintf(key, "IDMAP/UID2SID/%d", (int)uid);
-
-	state.sid = sid;
-	state.expired = expired;
-	state.ret = false;
-
-	gencache_parse(key, idmap_cache_xid2sid_parser, &state);
-	return state.ret;
-}
-
-/**
- * Find a gid2sid mapping
- * @param[in] gid		the gid to map
- * @param[out] sid		where to put the result
- * @param[out] expired		is the cache entry expired?
- * @retval Was anything in the cache at all?
- *
- * If "is_null_sid(sid)", this was a negative mapping.
- */
-
-bool idmap_cache_find_gid2sid(gid_t gid, struct dom_sid *sid, bool *expired)
-{
-	fstring key;
-	struct idmap_cache_xid2sid_state state;
-
-	fstr_sprintf(key, "IDMAP/GID2SID/%d", (int)gid);
-
-	state.sid = sid;
-	state.expired = expired;
-	state.ret = false;
-
-	gencache_parse(key, idmap_cache_xid2sid_parser, &state);
-	return state.ret;
-}
-
 /**
  * Find a xid2sid mapping
  * @param[in] id		the unix id to map
@@ -327,11 +280,12 @@ void idmap_cache_set_sid2unixid(const struct dom_sid *sid, struct unixid *unix_i
 {
 	time_t now = time(NULL);
 	time_t timeout;
-	fstring sidstr, key, value;
+	fstring key, value;
 
 	if (!is_null_sid(sid)) {
+		struct dom_sid_buf sidstr;
 		fstr_sprintf(key, "IDMAP/SID2XID/%s",
-			     sid_to_fstring(sidstr, sid));
+			     dom_sid_str_buf(sid, &sidstr));
 		switch (unix_id->type) {
 		case ID_TYPE_UID:
 			fstr_sprintf(value, "%d:U", (int)unix_id->id);
@@ -451,6 +405,7 @@ bool idmap_cache_del_sid(const struct dom_sid *sid)
 	bool ret = true;
 	bool expired;
 	struct unixid id;
+	struct dom_sid_buf sidbuf;
 	const char *sid_key;
 
 	if (!idmap_cache_find_sid2unixid(sid, &id, &expired)) {
@@ -475,7 +430,7 @@ bool idmap_cache_del_sid(const struct dom_sid *sid)
 		}
 	}
 
-	sid_key = key_sid2xid_str(mem_ctx, dom_sid_string(mem_ctx, sid));
+	sid_key = key_sid2xid_str(mem_ctx, dom_sid_str_buf(sid, &sidbuf));
 	if (sid_key == NULL) {
 		return false;
 	}
