@@ -30,9 +30,8 @@
 #include "lib/param/param.h"
 #include "librpc/crypto/gse.h"
 #include "auth/gensec/gensec.h"
-#include "auth/gensec/gensec_internal.h" /* TODO: remove this */
 #include "../libcli/auth/spnego.h"
-#include "krb5_errs.h"
+#include "lib/util/asn1.h"
 
 #ifdef HAVE_KRB5
 
@@ -40,68 +39,46 @@
 
 struct smb_krb5_context;
 
-/****************************************************************
-Callback to get the PAC_LOGON_INFO from the blob
-****************************************************************/
-static NTSTATUS kerberos_fetch_pac(struct auth4_context *auth_ctx,
-				   TALLOC_CTX *mem_ctx,
-				   struct smb_krb5_context *smb_krb5_context,
-				   DATA_BLOB *pac_blob,
-				   const char *princ_name,
-				   const struct tsocket_address *remote_address,
-				   uint32_t session_info_flags,
-				   struct auth_session_info **session_info)
+/*
+  generate a krb5 GSS-API wrapper packet given a ticket
+*/
+static DATA_BLOB spnego_gen_krb5_wrap(
+	TALLOC_CTX *ctx, const DATA_BLOB ticket, const uint8_t tok_id[2])
 {
-	TALLOC_CTX *tmp_ctx;
-	struct PAC_DATA *pac_data = NULL;
-	struct PAC_DATA_CTR *pac_data_ctr = NULL;
-	NTSTATUS status = NT_STATUS_INTERNAL_ERROR;
+	ASN1_DATA *data;
+	DATA_BLOB ret = data_blob_null;
 
-	tmp_ctx = talloc_new(mem_ctx);
-	if (!tmp_ctx) {
-		return NT_STATUS_NO_MEMORY;
+	data = asn1_init(talloc_tos(), ASN1_MAX_TREE_DEPTH);
+	if (data == NULL) {
+		return data_blob_null;
 	}
 
-	if (pac_blob != NULL) {
-		status = kerberos_decode_pac(tmp_ctx,
-					     *pac_blob,
-					     NULL,
-					     NULL,
-					     NULL,
-					     NULL,
-					     0,
-					     &pac_data);
-		if (!NT_STATUS_IS_OK(status)) {
-			goto done;
+	if (!asn1_push_tag(data, ASN1_APPLICATION(0))) goto err;
+	if (!asn1_write_OID(data, OID_KERBEROS5)) goto err;
+
+	if (!asn1_write(data, tok_id, 2)) goto err;
+	if (!asn1_write(data, ticket.data, ticket.length)) goto err;
+	if (!asn1_pop_tag(data)) goto err;
+
+	if (!asn1_extract_blob(data, ctx, &ret)) {
+		goto err;
+	}
+
+	asn1_free(data);
+	data = NULL;
+
+  err:
+
+	if (data != NULL) {
+		if (asn1_has_error(data)) {
+			DEBUG(1, ("Failed to build krb5 wrapper at offset %d\n",
+				  (int)asn1_current_ofs(data)));
 		}
 
-		pac_data_ctr = talloc(mem_ctx, struct PAC_DATA_CTR);
-		if (pac_data_ctr == NULL) {
-			status = NT_STATUS_NO_MEMORY;
-			goto done;
-		}
-
-		talloc_set_name_const(pac_data_ctr, "struct PAC_DATA_CTR");
-
-		pac_data_ctr->pac_data = talloc_steal(pac_data_ctr, pac_data);
-		pac_data_ctr->pac_blob = data_blob_talloc(pac_data_ctr,
-							  pac_blob->data,
-							  pac_blob->length);
-
-		auth_ctx->private_data = talloc_steal(auth_ctx, pac_data_ctr);
+		asn1_free(data);
 	}
 
-	*session_info = talloc_zero(mem_ctx, struct auth_session_info);
-	if (!*session_info) {
-		status = NT_STATUS_NO_MEMORY;
-		goto done;
-	}
-	status = NT_STATUS_OK;
-
-done:
-	TALLOC_FREE(tmp_ctx);
-
-	return status;
+	return ret;
 }
 
 /*
@@ -122,13 +99,18 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 			     time_t renewable_time,
 			     const char *impersonate_princ_s,
 			     const char *local_service,
+			     char **_canon_principal,
+			     char **_canon_realm,
 			     struct PAC_DATA_CTR **_pac_data_ctr)
 {
 	krb5_error_code ret;
 	NTSTATUS status = NT_STATUS_INVALID_PARAMETER;
-	DATA_BLOB tkt, tkt_wrapped, ap_rep, sesskey1;
+	DATA_BLOB tkt = data_blob_null;
+	DATA_BLOB tkt_wrapped = data_blob_null;
+	DATA_BLOB ap_rep = data_blob_null;
+	DATA_BLOB sesskey1 = data_blob_null;
 	const char *auth_princ = NULL;
-	const char *cc = "MEMORY:kerberos_return_pac";
+	const char *cc = NULL;
 	struct auth_session_info *session_info;
 	struct gensec_security *gensec_server_context;
 	const struct gensec_security_ops **backends;
@@ -137,6 +119,10 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 	struct auth4_context *auth_context;
 	struct loadparm_context *lp_ctx;
 	struct PAC_DATA_CTR *pac_data_ctr = NULL;
+	char *canon_principal = NULL;
+	char *canon_realm = NULL;
+	krb5_context ctx = NULL;
+	krb5_ccache ccid = NULL;
 
 	TALLOC_CTX *tmp_ctx = talloc_new(mem_ctx);
 	NT_STATUS_HAVE_NO_MEMORY(tmp_ctx);
@@ -146,11 +132,38 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 	ZERO_STRUCT(sesskey1);
 
 	if (!name || !pass) {
-		return NT_STATUS_INVALID_PARAMETER;
+		status = NT_STATUS_INVALID_PARAMETER;
+		goto out;
+	}
+
+	if (_canon_principal != NULL) {
+		*_canon_principal = NULL;
+	}
+
+	if (_canon_realm != NULL) {
+		*_canon_realm = NULL;
 	}
 
 	if (cache_name) {
 		cc = cache_name;
+	} else {
+		char *ccname = NULL;
+
+		ret = smb_krb5_init_context_common(&ctx);
+		if (ret != 0) {
+			status = krb5_to_nt_status(ret);
+			goto out;
+		}
+
+		ret = smb_krb5_cc_new_unique_memory(ctx,
+						    tmp_ctx,
+						    &ccname,
+						    &ccid);
+		if (ret != 0) {
+			status = krb5_to_nt_status(ret);
+			goto out;
+		}
+		cc = ccname;
 	}
 
 	if (!strchr_m(name, '@')) {
@@ -170,6 +183,9 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 					  request_pac,
 					  add_netbios_addr,
 					  renewable_time,
+					  tmp_ctx,
+					  &canon_principal,
+					  &canon_realm,
 					  &status);
 	if (ret) {
 		DEBUG(1,("kinit failed for '%s' with: %s (%d)\n",
@@ -195,7 +211,8 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 
 	if (expire_time && renew_till_time &&
 	    (*expire_time == 0) && (*renew_till_time == 0)) {
-		return NT_STATUS_INVALID_LOGON_TYPE;
+		status = NT_STATUS_INVALID_LOGON_TYPE;
+		goto out;
 	}
 
 	ret = ads_krb5_cli_get_ticket(mem_ctx,
@@ -225,12 +242,11 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 		goto out;
 	}
 
-	auth_context = talloc_zero(tmp_ctx, struct auth4_context);
+	auth_context = auth4_context_for_PAC_DATA_CTR(tmp_ctx);
 	if (auth_context == NULL) {
 		status = NT_STATUS_NO_MEMORY;
 		goto out;
 	}
-	auth_context->generate_session_info_pac = kerberos_fetch_pac;
 
 	lp_ctx = loadparm_init_s3(tmp_ctx, loadparm_s3_helpers());
 	if (lp_ctx == NULL) {
@@ -256,7 +272,7 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 
 	gensec_init();
 
-	backends[idx++] = &gensec_gse_krb5_security_ops;
+	backends[idx++] = gensec_gse_security_by_oid(GENSEC_OID_KERBEROS5);
 
 	status = gensec_server_start(tmp_ctx, gensec_settings,
 					auth_context, &gensec_server_context);
@@ -286,7 +302,7 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 		goto out;
 	}
 
-	/* Now return the PAC information to the callers.  We ingore
+	/* Now return the PAC information to the callers.  We ignore
 	 * the session_info and instead pick out the PAC via the
 	 * private_data on the auth_context */
 	status = gensec_session_info(gensec_server_context, tmp_ctx, &session_info);
@@ -295,8 +311,7 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 		goto out;
 	}
 
-	pac_data_ctr = talloc_get_type_abort(gensec_server_context->auth_context->private_data,
-					     struct PAC_DATA_CTR);
+	pac_data_ctr = auth4_context_get_PAC_DATA_CTR(auth_context, mem_ctx);
 	if (pac_data_ctr == NULL) {
 		DEBUG(1,("no PAC\n"));
 		status = NT_STATUS_INVALID_PARAMETER;
@@ -304,12 +319,23 @@ NTSTATUS kerberos_return_pac(TALLOC_CTX *mem_ctx,
 	}
 
 	*_pac_data_ctr = talloc_move(mem_ctx, &pac_data_ctr);
+	if (_canon_principal != NULL) {
+		*_canon_principal = talloc_move(mem_ctx, &canon_principal);
+	}
+	if (_canon_realm != NULL) {
+		*_canon_realm = talloc_move(mem_ctx, &canon_realm);
+	}
 
 out:
-	talloc_free(tmp_ctx);
-	if (cc != cache_name) {
-		ads_kdestroy(cc);
+	if (ccid != NULL) {
+		krb5_cc_destroy(ctx, ccid);
+		ccid = NULL;
 	}
+	if (ctx != NULL) {
+		krb5_free_context(ctx);
+		ctx = NULL;
+	}
+	talloc_free(tmp_ctx);
 
 	data_blob_free(&tkt);
 	data_blob_free(&ap_rep);
