@@ -26,6 +26,7 @@
 #define __LIBSMBCLIENT_INTERNAL__
 #include "libsmbclient.h"
 #include "libsmb_internal.h"
+#include "lib/param/param.h"
 
 
 /** Get the netbios name used for making connections */
@@ -94,7 +95,7 @@ smbc_setDebug(SMBCCTX *c, int debug)
 	TALLOC_CTX *frame = talloc_stackframe();
 	snprintf(buf, sizeof(buf), "%d", debug);
         c->debug = debug;
-	lp_set_cmdline("log level", buf);
+	lpcfg_set_cmdline(c->internal->lp_ctx, "log level", buf);
 	TALLOC_FREE(frame);
 }
 
@@ -534,16 +535,34 @@ smbc_setOptionProtocols(SMBCCTX *c,
 	bool ok = true;
 
 	if (min_proto != NULL) {
-		ok = lp_set_cmdline("client min protocol", min_proto);
+		ok = lpcfg_set_cmdline(c->internal->lp_ctx,
+				       "client min protocol",
+				       min_proto);
 	}
 
 	if (max_proto != NULL) {
-		ok &= lp_set_cmdline("client max protocol", max_proto);
+		ok &= lpcfg_set_cmdline(c->internal->lp_ctx,
+					"client max protocol",
+					max_proto);
 	}
 
 	return ok;
 }
 
+/** Get whether to enable POSIX extensions if available */
+smbc_bool
+smbc_getOptionPosixExtensions(SMBCCTX *c)
+{
+	return c->internal->posix_extensions;
+}
+
+/** Set whether to enable POSIX extensions if available */
+void
+smbc_setOptionPosixExtensions(SMBCCTX *c, smbc_bool b)
+{
+	c->internal->posix_extensions = b;
+}
+
 /** Get the function for obtaining authentication data */
 smbc_get_auth_data_fn
 smbc_getFunctionAuthData(SMBCCTX *c)
@@ -1054,6 +1073,18 @@ smbc_setFunctionGetxattr(SMBCCTX *c, smbc_getxattr_fn fn)
         c->getxattr = fn;
 }
 
+void
+smbc_setFunctionFGetxattr(SMBCCTX *c, smbc_fgetxattr_fn fn)
+{
+        c->internal->posix_emu.fgetxattr_fn = fn;
+}
+
+smbc_fgetxattr_fn
+smbc_getFunctionFGetxattr(SMBCCTX *c)
+{
+        return c->internal->posix_emu.fgetxattr_fn;
+}
+
 smbc_removexattr_fn
 smbc_getFunctionRemovexattr(SMBCCTX *c)
 {
