@@ -22,6 +22,7 @@
 
 #include "includes.h"
 #include "nmbd/nmbd.h"
+#include "lib/util/string_wrappers.h"
 
 /* This is our local master browser list database. */
 extern struct browse_cache_record *lmb_browserlist;
@@ -194,7 +195,7 @@ static void domain_master_node_status_success(struct subnet_record *subrec,
   /* Go through the list of names found at answers->rdata and look for
      the first SERVER<0x20> name. */
 
-	if(answers->rdata != NULL) {
+	if (answers->rdlength > 0) {
 		char *p = answers->rdata;
 		int numnames = CVAL(p, 0);
 
@@ -202,7 +203,7 @@ static void domain_master_node_status_success(struct subnet_record *subrec,
 
 		while (numnames--) {
 			unstring qname;
-			uint16 nb_flags;
+			uint16_t nb_flags;
 			int name_type;
 
 			pull_ascii_nstring(qname, sizeof(qname), p);
@@ -273,7 +274,7 @@ static void find_domain_master_name_query_success(struct subnet_record *subrec,
 	 * address we do a Node status request, and look for the first
 	 * NAME<0x20> in the response, and take that as the server name.
 	 * We also keep a cache of the Domain Master Browser name for this
-	 * workgroup in the Workgroup struct, so that if the same IP addess
+	 * workgroup in the Workgroup struct, so that if the same IP address
 	 * is returned every time, we don't need to do the node status
 	 * request.
 	 */
@@ -402,7 +403,6 @@ static void get_domain_master_name_node_status_success(struct subnet_record *sub
                                               struct res_rec *answers,
                                               struct in_addr from_ip)
 {
-	struct work_record *work;
 	unstring server_name;
 
 	server_name[0] = 0;
@@ -417,7 +417,7 @@ static void get_domain_master_name_node_status_success(struct subnet_record *sub
 	 * the first WORKGROUP<0x1b> name.
 	 */
 
-	if(answers->rdata != NULL) {
+	if (answers->rdlength > 0) {
 		char *p = answers->rdata;
 		int numnames = CVAL(p, 0);
 
@@ -425,7 +425,7 @@ static void get_domain_master_name_node_status_success(struct subnet_record *sub
 
 		while (numnames--) {
 			unstring qname;
-			uint16 nb_flags;
+			uint16_t nb_flags;
 			int name_type;
 
 			pull_ascii_nstring(qname, sizeof(qname), p);
@@ -443,6 +443,8 @@ static void get_domain_master_name_node_status_success(struct subnet_record *sub
 			}
 
 			if(!(nb_flags & NB_GROUP) && (name_type == 0x1b)) {
+				struct work_record *work;
+
 				if( DEBUGLVL( 5 ) ) {
 					dbgtext( "get_domain_master_name_node_status_success:\n" );
 					dbgtext( "%s(%s) ", server_name, inet_ntoa(from_ip) );
@@ -455,13 +457,16 @@ static void get_domain_master_name_node_status_success(struct subnet_record *sub
 				 * to the workgroup list on the unicast_subnet.
 				 */
 
-				if((work = find_workgroup_on_subnet( subrec, qname)) == NULL) {
+				work = find_workgroup_on_subnet( subrec, qname);
+				if (work == NULL) {
 					struct nmb_name nmbname;
 					/* 
 					 * Add it - with an hour in the cache.
 					 */
-					if(!(work= create_workgroup_on_subnet(subrec, qname, 60*60)))
+					work = create_workgroup_on_subnet(subrec, qname, 60*60);
+					if (work == NULL) {
 						return;
+					}
 
 					/* remember who the master is */
 					strlcpy(work->local_master_browser_name,
@@ -625,7 +630,7 @@ void sync_all_dmbs(time_t t)
 {
 	static time_t lastrun = 0;
 	struct work_record *work;
-	int count=0;
+	size_t count=0;
 
 	/* Only do this if we are using a WINS server. */
 	if(we_are_a_wins_client() == False)
