@@ -1,20 +1,20 @@
-/* 
+/*
    Unix SMB/Netbios implementation.
 
    Virtual lp system for printer testing
 
    Copyright (C) Tim Potter 2000
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -29,7 +29,7 @@
 #undef malloc
 #endif
 
-#define PRINT_FIRSTJOB "100"
+#define PRINT_FIRSTJOB 100
 
 static TDB_CONTEXT *tdb;
 
@@ -53,7 +53,7 @@ static void usage(void)
 
 /* Return an array of vlp jobs that is the printer queue */
 
-static void get_job_list(char *printer, struct vlp_job **job_list, 
+static void get_job_list(char *printer, struct vlp_job **job_list,
 			 int *num_jobs)
 {
 	fstring keystr;
@@ -68,7 +68,7 @@ static void get_job_list(char *printer, struct vlp_job **job_list,
 
 /* Store an array of vl jobs for the queue */
 
-static void set_job_list(char *printer, struct vlp_job *job_list, 
+static void set_job_list(char *printer, struct vlp_job *job_list,
 			 int num_jobs)
 {
 	fstring keystr;
@@ -97,10 +97,11 @@ static int next_jobnum(char *printer)
 	/* Create next job index if none exists */
 
 	if (jobnum == -1) {
-		jobnum = atoi(PRINT_FIRSTJOB);
+		jobnum = PRINT_FIRSTJOB;
+	} else {
+		jobnum++;
 	}
 
-	jobnum++;
 	tdb_store_int32(tdb, keystr, jobnum);
 
 	tdb_unlock_bystring(tdb, keystr);
@@ -111,10 +112,9 @@ static int next_jobnum(char *printer)
 static void set_printer_status(char *printer, int status)
 {
 	fstring keystr;
-	int result;
 
 	slprintf(keystr, sizeof(keystr) - 1, "STATUS/%s", printer);
-	result = tdb_store_int32(tdb, keystr, status);
+	tdb_store_int32(tdb, keystr, status);
 }
 
 static int get_printer_status(char *printer)
@@ -141,7 +141,7 @@ static int lpq_command(int argc, char **argv)
 {
 	char *printer;
 	struct vlp_job *job_list = NULL;
-	int i, num_jobs, job_count = 0;
+	int i, num_jobs;
 
 	if (argc != 2) {
 		printf("Usage: lpq <printername>\n");
@@ -172,12 +172,11 @@ static int lpq_command(int argc, char **argv)
 	for (i = 0; i < num_jobs; i++) {
 		if (job_list[i].deleted) continue;
 		printf("%d\t%d\t%d\t%ld\t%s\t%s\n", job_list[i].jobid,
-		       job_list[i].size, 
-		       (i == 0 && job_list[i].status == LPQ_QUEUED) ? 
+		       job_list[i].size,
+		       (i == 0 && job_list[i].status == LPQ_QUEUED) ?
 		       LPQ_SPOOLING : job_list[i].status,
-		       (long int)job_list[i].submit_time, job_list[i].owner, 
+		       (long int)job_list[i].submit_time, job_list[i].owner,
 		       job_list[i].jobname);
-		job_count++;
 	}
 
 	free(job_list);
@@ -394,6 +393,7 @@ int main(int argc, char **argv)
 		return 1;
 	}
 
+	/* FIXME: We should *never* open a tdb without logging! */
 	if (!(tdb = tdb_open(printdb_path, 0, 0, O_RDWR | O_CREAT,
 			     0666))) {
 		printf("%s: unable to open %s\n", argv[0], printdb_path);
@@ -405,6 +405,10 @@ int main(int argc, char **argv)
 	chmod(printdb_path, 0666);
 
 	/* Do commands */
+	if (argc < 3) {
+		usage();
+		return 1;
+	}
 
 	if (strcmp(argv[2], "lpq") == 0) {
 		return lpq_command(argc - 2, &argv[2]);
