@@ -4,7 +4,7 @@
 #
 # Copyright (C) 2011 Michael Adam <obnox@samba.org>
 
-incdir=`dirname $0`/../../../testprogs/blackbox
+incdir=$(dirname $0)/../../../testprogs/blackbox
 . $incdir/subunit.sh
 
 failed=0
@@ -20,16 +20,15 @@ test_success()
 }
 
 testit "success" \
-	test_success || \
-	failed=`expr $failed + 1`
+	test_success ||
+	failed=$(expr $failed + 1)
 
 testit "failure" \
-	test_failure || \
-	failed=`expr $failed + 1`
+	test_failure ||
+	failed=$(expr $failed + 1)
 
 testit "success" \
-	test_success || \
-	failed=`expr $failed + 1`
+	test_success ||
+	failed=$(expr $failed + 1)
 
 testok $0 $failed
-
