@@ -51,9 +51,14 @@ static void disk_norm(uint64_t *bsize, uint64_t *dfree, uint64_t *dsize)
  Return number of 1K blocks available on a path and total number.
 ****************************************************************************/
 
-uint64_t sys_disk_free(connection_struct *conn, struct smb_filename *fname,
-		       uint64_t *bsize, uint64_t *dfree, uint64_t *dsize)
+static uint64_t sys_disk_free(connection_struct *conn,
+			      struct smb_filename *fname,
+			      uint64_t *bsize,
+			      uint64_t *dfree,
+			      uint64_t *dsize)
 {
+	const struct loadparm_substitution *lp_sub =
+		loadparm_s3_global_substitution();
 	uint64_t dfree_retval;
 	uint64_t dfree_q = 0;
 	uint64_t bsize_q = 0;
@@ -69,27 +74,19 @@ uint64_t sys_disk_free(connection_struct *conn, struct smb_filename *fname,
 	 * If external disk calculation specified, use it.
 	 */
 
-	dfree_command = lp_dfree_command(talloc_tos(), SNUM(conn));
+	dfree_command = lp_dfree_command(talloc_tos(), lp_sub, SNUM(conn));
 	if (dfree_command && *dfree_command) {
 		const char *p;
 		char **lines = NULL;
 		char **argl = NULL;
 
-		argl = talloc_zero_array(talloc_tos(),
-					char *,
-					3);
+		argl = str_list_make_empty(talloc_tos());
+		str_list_add_printf(&argl, "%s", dfree_command);
+		str_list_add_printf(&argl, "%s", path);
 		if (argl == NULL) {
 			return (uint64_t)-1;
 		}
 
-		argl[0] = talloc_strdup(argl, dfree_command);
-		if (argl[0] == NULL) {
-			TALLOC_FREE(argl);
-			return (uint64_t)-1;
-		}
-		argl[1] = path;
-		argl[2] = NULL;
-
 		DBG_NOTICE("Running command '%s %s'\n",
 			dfree_command,
 			path);
@@ -183,7 +180,7 @@ dfree_done:
  Depending on the file system layout and file system features, the free space
  information can be different for different sub directories underneath a SMB
  share. Store the cache information in memcache using the query path as the
- key to accomodate this.
+ key to accommodate this.
 ****************************************************************************/
 
 struct dfree_cached_info {
