@@ -30,7 +30,7 @@
   for simplicity, we only allow ascii characters in 8.3 names
  */
 
- /* hash alghorithm changed to FNV1 by idra@samba.org (Simo Sorce).
+ /* hash algorithm changed to FNV1 by idra@samba.org (Simo Sorce).
   * see http://www.isthe.com/chongo/tech/comp/fnv/index.html for a
   * discussion on Fowler / Noll / Vo (FNV) Hash by one of it's authors
   */
@@ -101,8 +101,10 @@ static const char basechars[36] = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
 
 /* the list of reserved dos names - all of these are illegal */
 static const char * const reserved_names[] =
-{ "AUX", "LOCK$", "CON", "COM1", "COM2", "COM3", "COM4",
-  "LPT1", "LPT2", "LPT3", "NUL", "PRN", NULL };
+{ "AUX", "CON",
+  "COM1", "COM2", "COM3", "COM4", "COM5", "COM6", "COM7", "COM8", "COM9",
+  "LPT1", "LPT2", "LPT3", "LPT4", "LPT5", "LPT6", "LPT7", "LPT8", "LPT9",
+  "NUL", "PRN", NULL };
 
 #define DYNAMIC_MANGLE_TABLES 0
 
@@ -441,10 +443,8 @@ static bool is_8_3(const char *name, bool check_case, bool allow_wildcards, cons
 	char *dot_p;
 
 	/* as a special case, the names '.' and '..' are allowable 8.3 names */
-	if (name[0] == '.') {
-		if (!name[1] || (name[1] == '.' && !name[2])) {
-			return True;
-		}
+	if (ISDOT(name) || (ISDOTDOT(name))) {
+		return true;
 	}
 
 	/* the simplest test is on the overall length of the
@@ -575,7 +575,7 @@ static bool lookup_name_from_8_3(TALLOC_CTX *ctx,
 	TALLOC_FREE(prefix);
 
 	if (!*pp_out) {
-		M_DEBUG(0,("talloc_fail"));
+		M_DEBUG(0,("talloc_fail\n"));
 		return False;
 	}
 
