@@ -34,7 +34,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 					       struct smbd_smb2_request *smb2req,
 					       struct files_struct *in_fsp,
 					       DATA_BLOB in_data,
-					       uint64_t in_offset,
+					       off_t in_offset,
 					       uint32_t in_flags);
 static NTSTATUS smbd_smb2_write_recv(struct tevent_req *req,
 				     uint32_t *out_count);
@@ -48,7 +48,7 @@ NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 	uint16_t in_data_offset;
 	uint32_t in_data_length;
 	DATA_BLOB in_data_buffer;
-	uint64_t in_offset;
+	off_t in_offset;
 	uint64_t in_file_id_persistent;
 	uint64_t in_file_id_volatile;
 	struct files_struct *in_fsp;
@@ -65,7 +65,7 @@ NTSTATUS smbd_smb2_request_process_write(struct smbd_smb2_request *req)
 
 	in_data_offset		= SVAL(inbody, 0x02);
 	in_data_length		= IVAL(inbody, 0x04);
-	in_offset		= BVAL(inbody, 0x08);
+	in_offset		= PULL_LE_I64(inbody, 0x08);
 	in_file_id_persistent	= BVAL(inbody, 0x10);
 	in_file_id_volatile	= BVAL(inbody, 0x18);
 	in_flags		= IVAL(inbody, 0x2C);
@@ -178,7 +178,7 @@ struct smbd_smb2_write_state {
 	files_struct *fsp;
 	bool write_through;
 	uint32_t in_length;
-	uint64_t in_offset;
+	off_t in_offset;
 	uint32_t out_count;
 };
 
@@ -266,7 +266,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 					       struct smbd_smb2_request *smb2req,
 					       struct files_struct *fsp,
 					       DATA_BLOB in_data,
-					       uint64_t in_offset,
+					       off_t in_offset,
 					       uint32_t in_flags)
 {
 	NTSTATUS status;
@@ -298,7 +298,7 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 	DEBUG(10,("smbd_smb2_write: %s - %s\n",
 		  fsp_str_dbg(fsp), fsp_fnum_dbg(fsp)));
 
-	smbreq = smbd_smb2_fake_smb_request(smb2req);
+	smbreq = smbd_smb2_fake_smb_request(smb2req, fsp);
 	if (tevent_req_nomem(smbreq, req)) {
 		return tevent_req_post(req, ev);
 	}
@@ -346,6 +346,18 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 		return tevent_req_post(req, ev);
 	}
 
+	if (state->in_offset == VFS_PWRITE_APPEND_OFFSET &&
+	    !fsp->fsp_flags.posix_append)
+	{
+		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
+		return tevent_req_post(req, ev);
+	} else if (fsp->fsp_flags.posix_append &&
+		   state->in_offset != VFS_PWRITE_APPEND_OFFSET)
+	{
+		tevent_req_nterror(req, NT_STATUS_INVALID_PARAMETER);
+		return tevent_req_post(req, ev);
+	}
+
 	/* Try and do an asynchronous write. */
 	status = schedule_aio_smb2_write(conn,
 					smbreq,
@@ -375,7 +387,6 @@ static struct tevent_req *smbd_smb2_write_send(TALLOC_CTX *mem_ctx,
 				in_offset,
 				in_data.length,
 				WRITE_LOCK,
-				lp_posix_cifsu_locktype(fsp),
 				&lock);
 
 	if (!SMB_VFS_STRICT_LOCK_CHECK(conn, fsp, &lock)) {
