@@ -79,7 +79,7 @@ void print_registry_value(const struct registry_value *valvalue, bool raw)
 		break;
 	}
 	case REG_MULTI_SZ: {
-		uint32 j;
+		uint32_t j;
 		const char **a;
 
 		if (!pull_reg_multi_sz(talloc_tos(), &valvalue->data, &a)) {
@@ -135,11 +135,11 @@ WERROR split_hive_key(TALLOC_CTX *ctx, const char *path, char **hivename,
 	const char *tmp_subkeyname;
 
 	if ((path == NULL) || (hivename == NULL) || (subkeyname == NULL)) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	if (strlen(path) == 0) {
-		return WERR_INVALID_PARAM;
+		return WERR_INVALID_PARAMETER;
 	}
 
 	if (strchr(path, '\\') == NULL) {
@@ -149,7 +149,7 @@ WERROR split_hive_key(TALLOC_CTX *ctx, const char *path, char **hivename,
 	}
 
 	if (*hivename == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	/* strip trailing '\\' chars */
@@ -170,7 +170,7 @@ WERROR split_hive_key(TALLOC_CTX *ctx, const char *path, char **hivename,
 	}
 	*subkeyname = talloc_strdup(ctx, tmp_subkeyname);
 	if (*subkeyname == NULL) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 
 	return WERR_OK;
