@@ -1,4 +1,4 @@
-/* 
+/*
    ldb database library
 
    Copyright (C) Simo Sorce 2005-2008
@@ -8,12 +8,12 @@
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -37,6 +37,8 @@
 #include "dsdb/samdb/ldb_modules/util.h"
 #include "lib/ldb-samba/ldb_matching_rules.h"
 
+#undef strncasecmp
+
 /*
   TODO: if relax is not set then we need to reject the fancy RMD_* and
   DELETED extended DN codes
@@ -46,6 +48,7 @@
 struct extended_search_context {
 	struct ldb_module *module;
 	struct ldb_request *req;
+	struct ldb_parse_tree *tree;
 	struct ldb_dn *basedn;
 	struct ldb_dn *dn;
 	char *wellknown_object;
@@ -74,15 +77,15 @@ static int extended_final_callback(struct ldb_request *req, struct ldb_reply *ar
 	} else {
 		switch (ares->type) {
 		case LDB_REPLY_ENTRY:
-			
+
 			ret = ldb_module_send_entry(ac->req, ares->message, ares->controls);
 			break;
 		case LDB_REPLY_REFERRAL:
-			
+
 			ret = ldb_module_send_referral(ac->req, ares->referral);
 			break;
 		case LDB_REPLY_DONE:
-			
+
 			ret = ldb_module_done(ac->req, ares->controls,
 					      ares->response, ares->error);
 			break;
@@ -198,10 +201,10 @@ static int extended_base_callback(struct ldb_request *req, struct ldb_reply *are
 						      ldb_module_get_ctx(ac->module), ac->req,
 						      ac->basedn,
 						      ac->req->op.search.scope,
-						      ac->req->op.search.tree,
+						      ac->tree,
 						      ac->req->op.search.attrs,
 						      ac->req->controls,
-						      ac, extended_final_callback, 
+						      ac, extended_final_callback,
 						      ac->req);
 			LDB_REQ_SET_LOCATION(down_req);
 			break;
@@ -213,14 +216,14 @@ static int extended_base_callback(struct ldb_request *req, struct ldb_reply *are
 				return ldb_module_done(ac->req, NULL, NULL,
 						       LDB_ERR_OPERATIONS_ERROR);
 			}
-			
+
 			add_msg->dn = ac->basedn;
 
 			ret = ldb_build_add_req(&down_req,
 						ldb_module_get_ctx(ac->module), ac->req,
-						add_msg, 
+						add_msg,
 						ac->req->controls,
-						ac, extended_final_callback, 
+						ac, extended_final_callback,
 						ac->req);
 			LDB_REQ_SET_LOCATION(down_req);
 			break;
@@ -233,14 +236,14 @@ static int extended_base_callback(struct ldb_request *req, struct ldb_reply *are
 				return ldb_module_done(ac->req, NULL, NULL,
 						       LDB_ERR_OPERATIONS_ERROR);
 			}
-			
+
 			mod_msg->dn = ac->basedn;
 
 			ret = ldb_build_mod_req(&down_req,
 						ldb_module_get_ctx(ac->module), ac->req,
-						mod_msg, 
+						mod_msg,
 						ac->req->controls,
-						ac, extended_final_callback, 
+						ac, extended_final_callback,
 						ac->req);
 			LDB_REQ_SET_LOCATION(down_req);
 			break;
@@ -248,26 +251,26 @@ static int extended_base_callback(struct ldb_request *req, struct ldb_reply *are
 		case LDB_DELETE:
 			ret = ldb_build_del_req(&down_req,
 						ldb_module_get_ctx(ac->module), ac->req,
-						ac->basedn, 
+						ac->basedn,
 						ac->req->controls,
-						ac, extended_final_callback, 
+						ac, extended_final_callback,
 						ac->req);
 			LDB_REQ_SET_LOCATION(down_req);
 			break;
 		case LDB_RENAME:
 			ret = ldb_build_rename_req(&down_req,
 						   ldb_module_get_ctx(ac->module), ac->req,
-						   ac->basedn, 
+						   ac->basedn,
 						   ac->req->op.rename.newdn,
 						   ac->req->controls,
-						   ac, extended_final_callback, 
+						   ac, extended_final_callback,
 						   ac->req);
 			LDB_REQ_SET_LOCATION(down_req);
 			break;
 		default:
 			return ldb_module_done(ac->req, NULL, NULL, LDB_ERR_OPERATIONS_ERROR);
 		}
-		
+
 		if (ret != LDB_SUCCESS) {
 			return ldb_module_done(ac->req, NULL, NULL, ret);
 		}
@@ -280,7 +283,7 @@ static int extended_base_callback(struct ldb_request *req, struct ldb_reply *are
 
 
 /*
-  windows ldap searchs don't allow a baseDN with more
+  windows ldap searches don't allow a baseDN with more
   than one extended component, or an extended
   component and a string DN
 
@@ -420,7 +423,15 @@ static int extended_dn_filter_callback(struct ldb_parse_tree *tree, void *privat
 	guid_val = ldb_dn_get_extended_component(dn, "GUID");
 	sid_val  = ldb_dn_get_extended_component(dn, "SID");
 
-	if (!guid_val && !sid_val && (attribute->searchFlags & SEARCH_FLAG_ATTINDEX)) {
+	/*
+	 * Is the attribute indexed? By treating confidential attributes
+	 * as unindexed, we force searches to go through the unindexed
+	 * search path, avoiding observable timing differences.
+	 */
+	if (!guid_val && !sid_val &&
+	    (attribute->searchFlags & SEARCH_FLAG_ATTINDEX) &&
+	    !(attribute->searchFlags & SEARCH_FLAG_CONFIDENTIAL))
+	{
 		/* if it is indexed, then fixing the string DN will do
 		   no good here, as we will not find the attribute in
 		   the index. So for now fall through to a standard DN
@@ -509,15 +520,18 @@ static int extended_dn_filter_callback(struct ldb_parse_tree *tree, void *privat
 
 /*
   fix the parse tree to change any extended DN components to their
-  caconical form
+  canonical form
  */
 static int extended_dn_fix_filter(struct ldb_module *module,
 				  struct ldb_request *req,
-				  uint32_t default_dsdb_flags)
+				  uint32_t default_dsdb_flags,
+				  struct ldb_parse_tree **down_tree)
 {
 	struct extended_dn_filter_ctx *filter_ctx;
 	int ret;
 
+	*down_tree = NULL;
+
 	filter_ctx = talloc_zero(req, struct extended_dn_filter_ctx);
 	if (filter_ctx == NULL) {
 		return ldb_module_oom(module);
@@ -548,12 +562,12 @@ static int extended_dn_fix_filter(struct ldb_module *module,
 	filter_ctx->test_only = false;
 	filter_ctx->matched   = false;
 
-	req->op.search.tree = ldb_parse_tree_copy_shallow(req, req->op.search.tree);
-	if (req->op.search.tree == NULL) {
+	*down_tree = ldb_parse_tree_copy_shallow(req, req->op.search.tree);
+	if (*down_tree == NULL) {
 		return ldb_oom(ldb_module_get_ctx(module));
 	}
 
-	ret = ldb_parse_tree_walk(req->op.search.tree, extended_dn_filter_callback, filter_ctx);
+	ret = ldb_parse_tree_walk(*down_tree, extended_dn_filter_callback, filter_ctx);
 	if (ret != LDB_SUCCESS) {
 		talloc_free(filter_ctx);
 		return ret;
@@ -570,7 +584,8 @@ static int extended_dn_fix_filter(struct ldb_module *module,
 static int extended_dn_in_fix(struct ldb_module *module, struct ldb_request *req, struct ldb_dn *dn)
 {
 	struct extended_search_context *ac;
-	struct ldb_request *down_req;
+	struct ldb_request *down_req = NULL;
+	struct ldb_parse_tree *down_tree = NULL;
 	int ret;
 	struct ldb_dn *base_dn = NULL;
 	enum ldb_scope base_dn_scope = LDB_SCOPE_BASE;
@@ -593,7 +608,7 @@ static int extended_dn_in_fix(struct ldb_module *module, struct ldb_request *req
 	}
 
 	if (req->operation == LDB_SEARCH) {
-		ret = extended_dn_fix_filter(module, req, dsdb_flags);
+		ret = extended_dn_fix_filter(module, req, dsdb_flags, &down_tree);
 		if (ret != LDB_SUCCESS) {
 			return ret;
 		}
@@ -601,7 +616,25 @@ static int extended_dn_in_fix(struct ldb_module *module, struct ldb_request *req
 
 	if (!ldb_dn_has_extended(dn)) {
 		/* Move along there isn't anything to see here */
-		return ldb_next_request(module, req);
+		if (down_tree == NULL) {
+			down_req = req;
+		} else {
+			ret = ldb_build_search_req_ex(&down_req,
+						      ldb_module_get_ctx(module), req,
+						      req->op.search.base,
+						      req->op.search.scope,
+						      down_tree,
+						      req->op.search.attrs,
+						      req->controls,
+						      req, dsdb_next_callback,
+						      req);
+			if (ret != LDB_SUCCESS) {
+				return ret;
+			}
+			LDB_REQ_SET_LOCATION(down_req);
+		}
+
+		return ldb_next_request(module, down_req);
 	} else {
 		/* It looks like we need to map the DN */
 		const struct ldb_val *sid_val, *guid_val, *wkguid_val;
@@ -618,7 +651,7 @@ static int extended_dn_in_fix(struct ldb_module *module, struct ldb_request *req
 		/*
 		  prioritise the GUID - we have had instances of
 		  duplicate SIDs in the database in the
-		  ForeignSecurityPrinciples due to provision errors
+		  ForeignSecurityPrincipals due to provision errors
 		 */
 		if (guid_val) {
 			dsdb_flags |= DSDB_SEARCH_SEARCH_ALL_PARTITIONS;
@@ -685,13 +718,14 @@ static int extended_dn_in_fix(struct ldb_module *module, struct ldb_request *req
 		if (ac == NULL) {
 			return ldb_oom(ldb_module_get_ctx(module));
 		}
-		
+
 		ac->module = module;
 		ac->req = req;
+		ac->tree = (down_tree != NULL) ? down_tree : req->op.search.tree;
 		ac->dn = dn;
 		ac->basedn = NULL;  /* Filled in if the search finds the DN by SID/GUID etc */
 		ac->wellknown_object = wellknown_object;
-		
+
 		/* If the base DN was an extended DN (perhaps a well known
 		 * GUID) then search for that, so we can proceed with the original operation */
 
