@@ -26,7 +26,6 @@ import samba
 
 from samba.ndr import ndr_unpack, ndr_print
 from samba.dcerpc import misc
-from samba.dcerpc import security
 from samba.dcerpc import drsblobs
 from samba.dcerpc.drsuapi import *
 from samba.tests.password_test import PasswordCommon
@@ -60,7 +59,10 @@ class RestoredObjectAttributesBaseTestCase(samba.tests.TestCase):
     def GUID_string(self, guid):
         return get_string(self.samdb.schema_format_value("objectGUID", guid))
 
-    def search_guid(self, guid, attrs=["*"]):
+    def search_guid(self, guid, attrs=None):
+        if attrs is None:
+            attrs = ["*"]
+
         res = self.samdb.search(base="<GUID=%s>" % self.GUID_string(guid),
                                 scope=SCOPE_BASE, attrs=attrs,
                                 controls=["show_deleted:1"])
