@@ -37,7 +37,6 @@
 
 static bool samba_wdc_is_s4u2self_req(astgs_request_t r)
 {
-	krb5_kdc_configuration *config = kdc_request_get_config((kdc_request_t)r);
 	const KDC_REQ *req = kdc_request_get_req(r);
 	const PA_DATA *pa_for_user = NULL;
 
@@ -45,27 +44,6 @@ static bool samba_wdc_is_s4u2self_req(astgs_request_t r)
 		return false;
 	}
 
-	if (config->enable_fast && req->padata != NULL) {
-		const PA_DATA *pa_fx_fast = NULL;
-		int idx = 0;
-
-		pa_fx_fast = krb5_find_padata(req->padata->val,
-					      req->padata->len,
-					      KRB5_PADATA_FX_FAST,
-					      &idx);
-		if (pa_fx_fast != NULL) {
-			/*
-			 * We're in the outer request
-			 * with KRB5_PADATA_FX_FAST
-			 * if fast is enabled we'll
-			 * process the s4u2self
-			 * request only in the
-			 * inner request.
-			 */
-			return false;
-		}
-	}
-
 	if (req->padata != NULL) {
 		int idx = 0;
 
@@ -98,45 +76,29 @@ static krb5_error_code samba_wdc_get_pac(void *priv,
 {
 	krb5_context context = kdc_request_get_context((kdc_request_t)r);
 	TALLOC_CTX *mem_ctx;
-	DATA_BLOB *logon_blob = NULL;
-	DATA_BLOB *cred_ndr = NULL;
-	DATA_BLOB **cred_ndr_ptr = NULL;
-	DATA_BLOB _cred_blob = data_blob_null;
-	DATA_BLOB *cred_blob = NULL;
-	DATA_BLOB *upn_blob = NULL;
-	DATA_BLOB *pac_attrs_blob = NULL;
-	DATA_BLOB *requester_sid_blob = NULL;
-	const DATA_BLOB *client_claims_blob = NULL;
 	krb5_error_code ret;
-	NTSTATUS nt_status;
-	struct samba_kdc_entry *skdc_entry =
+	struct samba_kdc_entry *client_entry =
 		talloc_get_type_abort(client->context,
 		struct samba_kdc_entry);
 	const struct samba_kdc_entry *server_entry =
 		talloc_get_type_abort(server->context,
 		struct samba_kdc_entry);
-	bool is_krbtgt = krb5_principal_is_krbtgt(context, server->principal);
-	/* Only include resource groups in a service ticket. */
-	enum auth_group_inclusion group_inclusion;
+	struct samba_kdc_entry_pac device_pac_entry = {};
 	bool is_s4u2self = samba_wdc_is_s4u2self_req(r);
-	enum samba_asserted_identity asserted_identity =
-		(is_s4u2self) ?
-			SAMBA_ASSERTED_IDENTITY_SERVICE :
-			SAMBA_ASSERTED_IDENTITY_AUTHENTICATION_AUTHORITY;
-	const enum samba_claims_valid claims_valid = SAMBA_CLAIMS_VALID_INCLUDE;
-	const enum samba_compounded_auth compounded_auth = SAMBA_COMPOUNDED_AUTH_EXCLUDE;
+	uint32_t flags = 0;
 	struct authn_audit_info *server_audit_info = NULL;
-	NTSTATUS status = NT_STATUS_OK;
+	NTSTATUS reply_status = NT_STATUS_OK;
 
-	struct auth_user_info_dc *user_info_dc = NULL;
+	if (pac == NULL) {
+		return EINVAL;
+	}
 
-	/* Only include resource groups in a service ticket. */
-	if (is_krbtgt) {
-		group_inclusion = AUTH_EXCLUDE_RESOURCE_GROUPS;
-	} else if (server_entry->supported_enctypes & KERB_ENCTYPE_RESOURCE_SID_COMPRESSION_DISABLED) {
-		group_inclusion = AUTH_INCLUDE_RESOURCE_GROUPS;
-	} else {
-		group_inclusion = AUTH_INCLUDE_RESOURCE_GROUPS_COMPRESSED;
+	if (is_s4u2self) {
+		flags |= SAMBA_KDC_FLAG_PROTOCOL_TRANSITION;
+	}
+
+	if (kdc_request_get_pkinit_freshness_used(r)) {
+		flags |= SAMBA_KDC_FLAG_PKINIT_FRESHNESS_USED;
 	}
 
 	mem_ctx = talloc_named(client->context, 0, "samba_wdc_get_pac context");
@@ -144,120 +106,7 @@ static krb5_error_code samba_wdc_get_pac(void *priv,
 		return ENOMEM;
 	}
 
-	if (pk_reply_key != NULL) {
-		cred_ndr_ptr = &cred_ndr;
-	}
-
-	nt_status = samba_kdc_get_user_info_dc(mem_ctx,
-					       skdc_entry,
-					       asserted_identity,
-					       claims_valid,
-					       compounded_auth,
-					       &user_info_dc);
-	if (!NT_STATUS_IS_OK(nt_status)) {
-		talloc_free(mem_ctx);
-		return EINVAL;
-	}
-
-	/*
-	 * For an S4U2Self request, the authentication policy is not enforced.
-	 */
-	if (!is_s4u2self && authn_policy_restrictions_present(server_entry->server_policy)) {
-		ret = samba_kdc_allowed_to_authenticate_to(mem_ctx,
-							   server_entry->kdc_db_ctx->samdb,
-							   server_entry->kdc_db_ctx->lp_ctx,
-							   skdc_entry,
-							   user_info_dc,
-							   server_entry,
-							   &server_audit_info,
-							   &status);
-		if (server_audit_info != NULL) {
-			krb5_error_code ret2;
-
-			ret2 = hdb_samba4_set_steal_server_audit_info(r, server_audit_info);
-			if (ret == 0) {
-				ret = ret2;
-			}
-		}
-		if (!NT_STATUS_IS_OK(status)) {
-			krb5_error_code ret2;
-
-			ret2 = hdb_samba4_set_ntstatus(r, status, ret);
-			if (ret == 0) {
-				ret = ret2;
-			}
-		}
-		if (ret) {
-			talloc_free(mem_ctx);
-			return ret;
-		}
-	}
-
-	nt_status = samba_kdc_get_logon_info_blob(mem_ctx,
-						  user_info_dc,
-						  group_inclusion,
-						  &logon_blob);
-	if (!NT_STATUS_IS_OK(nt_status)) {
-		talloc_free(mem_ctx);
-		return EINVAL;
-	}
-
-	if (cred_ndr_ptr != NULL) {
-		nt_status = samba_kdc_get_cred_ndr_blob(mem_ctx,
-							skdc_entry,
-							cred_ndr_ptr);
-		if (!NT_STATUS_IS_OK(nt_status)) {
-			talloc_free(mem_ctx);
-			return EINVAL;
-		}
-	}
-
-	nt_status = samba_kdc_get_upn_info_blob(mem_ctx,
-						user_info_dc,
-						&upn_blob);
-	if (!NT_STATUS_IS_OK(nt_status)) {
-		talloc_free(mem_ctx);
-		return EINVAL;
-	}
-
-	if (is_krbtgt) {
-		nt_status = samba_kdc_get_pac_attrs_blob(mem_ctx,
-							 pac_attributes,
-							 &pac_attrs_blob);
-		if (!NT_STATUS_IS_OK(nt_status)) {
-			talloc_free(mem_ctx);
-			return EINVAL;
-		}
-
-		nt_status = samba_kdc_get_requester_sid_blob(mem_ctx,
-							     user_info_dc,
-							     &requester_sid_blob);
-		if (!NT_STATUS_IS_OK(nt_status)) {
-			talloc_free(mem_ctx);
-			return EINVAL;
-		}
-	}
-
-	nt_status = samba_kdc_get_claims_blob(mem_ctx,
-					      skdc_entry,
-					      &client_claims_blob);
-	if (!NT_STATUS_IS_OK(nt_status)) {
-		talloc_free(mem_ctx);
-		return EINVAL;
-	}
-
-	if (pk_reply_key != NULL && cred_ndr != NULL) {
-		ret = samba_kdc_encrypt_pac_credentials(context,
-							pk_reply_key,
-							cred_ndr,
-							mem_ctx,
-							&_cred_blob);
-		if (ret != 0) {
-			talloc_free(mem_ctx);
-			return ret;
-		}
-		cred_blob = &_cred_blob;
-	}
+	device_pac_entry = samba_kdc_get_device_pac(r);
 
 	ret = krb5_pac_init(context, pac);
 	if (ret != 0) {
@@ -265,31 +114,59 @@ static krb5_error_code samba_wdc_get_pac(void *priv,
 		return ret;
 	}
 
-	ret = samba_make_krb5_pac(context, logon_blob, cred_blob,
-				  upn_blob, pac_attrs_blob,
-				  requester_sid_blob, NULL,
-				  client_claims_blob, NULL, NULL,
-				  *pac);
+	ret = samba_kdc_get_pac(mem_ctx,
+				context,
+				server_entry->kdc_db_ctx,
+				flags,
+				client_entry,
+				server->principal,
+				server_entry,
+				device_pac_entry,
+				pk_reply_key,
+				pac_attributes,
+				*pac,
+				&server_audit_info,
+				&reply_status);
+	if (server_audit_info != NULL) {
+		krb5_error_code ret2;
+
+		ret2 = hdb_samba4_set_steal_server_audit_info(r, server_audit_info);
+		if (ret == 0) {
+			ret = ret2;
+		}
+	}
+	if (!NT_STATUS_IS_OK(reply_status)) {
+		krb5_error_code ret2;
+
+		ret2 = hdb_samba4_set_ntstatus(r, reply_status, ret);
+		if (ret == 0) {
+			ret = ret2;
+		}
+	}
+	if (ret) {
+		krb5_pac_free(context, *pac);
+		*pac = NULL;
+		talloc_free(mem_ctx);
+		return ret;
+	}
 
 	talloc_free(mem_ctx);
 	return ret;
 }
 
 static krb5_error_code samba_wdc_verify_pac2(astgs_request_t r,
+					     krb5_const_principal client_principal,
 					     const hdb_entry *delegated_proxy,
 					     const hdb_entry *client,
 					     const hdb_entry *krbtgt,
 					     const krb5_pac pac,
-					     krb5_cksumtype ctype,
-					     const hdb_entry *device,
-					     krb5_const_pac *device_pac,
-					     krb5_boolean *is_trusted_out)
+					     krb5_cksumtype ctype)
 {
 	krb5_context context = kdc_request_get_context((kdc_request_t)r);
 	struct samba_kdc_entry *client_skdc_entry = NULL;
-	struct samba_kdc_entry *device_skdc_entry = NULL;
 	struct samba_kdc_entry *krbtgt_skdc_entry =
 		talloc_get_type_abort(krbtgt->context, struct samba_kdc_entry);
+	struct samba_kdc_entry_pac client_pac_entry = {};
 	TALLOC_CTX *mem_ctx = NULL;
 	krb5_error_code ret;
 	bool is_s4u2self = samba_wdc_is_s4u2self_req(r);
@@ -297,6 +174,10 @@ static krb5_error_code samba_wdc_verify_pac2(astgs_request_t r,
 	bool is_trusted = false;
 	uint32_t flags = 0;
 
+	if (pac == NULL) {
+		return EINVAL;
+	}
+
 	mem_ctx = talloc_named(NULL, 0, "samba_wdc_verify_pac2 context");
 	if (mem_ctx == NULL) {
 		return ENOMEM;
@@ -307,17 +188,12 @@ static krb5_error_code samba_wdc_verify_pac2(astgs_request_t r,
 							  struct samba_kdc_entry);
 	}
 
-	if (device != NULL) {
-		device_skdc_entry = talloc_get_type_abort(device->context,
-							  struct samba_kdc_entry);
-	}
-
 	/*
 	 * If the krbtgt was generated by an RODC, and we are not that
 	 * RODC, then we need to regenerate the PAC - we can't trust
 	 * it, and confirm that the RODC was permitted to print this ticket
 	 *
-	 * Becasue of the samba_kdc_validate_pac_blob() step we can be
+	 * Because of the samba_kdc_validate_pac_blob() step we can be
 	 * sure that the record in 'client' matches the SID in the
 	 * original PAC.
 	 */
@@ -326,6 +202,12 @@ static krb5_error_code samba_wdc_verify_pac2(astgs_request_t r,
 		goto out;
 	}
 
+	krb5_pac_set_trusted(pac, is_trusted);
+	client_pac_entry = samba_kdc_entry_pac(pac,
+					       client_principal,
+					       client_skdc_entry,
+					       krbtgt_skdc_entry);
+
 	if (is_s4u2self) {
 		flags |= SAMBA_KDC_FLAG_PROTOCOL_TRANSITION;
 	}
@@ -370,46 +252,32 @@ static krb5_error_code samba_wdc_verify_pac2(astgs_request_t r,
 				      NULL,
 				      &key->key);
 		if (ret != 0) {
-			DEBUG(1, ("PAC KDC signature failed to verify\n"));
+			DBG_WARNING("PAC KDC signature failed to verify\n");
 			goto out;
 		}
 
 		flags |= SAMBA_KDC_FLAG_CONSTRAINED_DELEGATION;
 	}
 
-	if (is_trusted) {
-		flags |= SAMBA_KDC_FLAG_KRBTGT_IS_TRUSTED;
-	}
-
-	if (is_in_db) {
-		flags |= SAMBA_KDC_FLAG_KRBTGT_IN_DB;
-	}
-
 	ret = samba_kdc_verify_pac(mem_ctx,
 				   context,
+				   krbtgt_skdc_entry->kdc_db_ctx,
 				   flags,
-				   client_skdc_entry,
-				   krbtgt_skdc_entry,
-				   device_skdc_entry,
-				   device_pac,
-				   pac);
+				   client_pac_entry,
+				   krbtgt_skdc_entry);
 	if (ret != 0) {
 		goto out;
 	}
 
-	if (is_trusted_out != NULL) {
-		*is_trusted_out = is_trusted;
-	}
-
 out:
 	talloc_free(mem_ctx);
 	return ret;
 }
 
-/* Resign (and reform, including possibly new groups) a PAC */
+/* Re-sign (and reform, including possibly new groups) a PAC */
 
 static krb5_error_code samba_wdc_reget_pac(void *priv, astgs_request_t r,
-					   krb5_const_principal _client_principal,
+					   krb5_const_principal client_principal,
 					   hdb_entry *delegated_proxy,
 					   krb5_const_pac delegated_proxy_pac,
 					   hdb_entry *client,
@@ -418,25 +286,28 @@ static krb5_error_code samba_wdc_reget_pac(void *priv, astgs_request_t r,
 					   krb5_pac *pac)
 {
 	krb5_context context = kdc_request_get_context((kdc_request_t)r);
-	const hdb_entry *device = kdc_request_get_explicit_armor_client(r);
-	const krb5_const_pac device_pac = kdc_request_get_explicit_armor_pac(r);
 	struct samba_kdc_entry *delegated_proxy_skdc_entry = NULL;
+	const struct samba_kdc_entry *delegated_proxy_krbtgt_entry = NULL;
 	krb5_const_principal delegated_proxy_principal = NULL;
+	struct samba_kdc_entry_pac delegated_proxy_pac_entry = {};
 	struct samba_kdc_entry *client_skdc_entry = NULL;
-	struct samba_kdc_entry *device_skdc_entry = NULL;
+	struct samba_kdc_entry_pac client_pac_entry = {};
+	struct samba_kdc_entry_pac device = {};
 	const struct samba_kdc_entry *server_skdc_entry =
 		talloc_get_type_abort(server->context, struct samba_kdc_entry);
 	const struct samba_kdc_entry *krbtgt_skdc_entry =
 		talloc_get_type_abort(krbtgt->context, struct samba_kdc_entry);
-	const struct samba_kdc_entry *client_krbtgt_skdc_entry = krbtgt_skdc_entry;
-	const struct samba_kdc_entry *device_krbtgt_skdc_entry = NULL;
 	TALLOC_CTX *mem_ctx = NULL;
 	krb5_pac new_pac = NULL;
 	struct authn_audit_info *server_audit_info = NULL;
 	krb5_error_code ret;
-	NTSTATUS status = NT_STATUS_OK;
+	NTSTATUS reply_status = NT_STATUS_OK;
 	uint32_t flags = 0;
 
+	if (pac == NULL) {
+		return EINVAL;
+	}
+
 	mem_ctx = talloc_named(NULL, 0, "samba_wdc_reget_pac context");
 	if (mem_ctx == NULL) {
 		return ENOMEM;
@@ -446,23 +317,31 @@ static krb5_error_code samba_wdc_reget_pac(void *priv, astgs_request_t r,
 		delegated_proxy_skdc_entry = talloc_get_type_abort(delegated_proxy->context,
 								   struct samba_kdc_entry);
 		delegated_proxy_principal = delegated_proxy->principal;
+
+		/*
+		 * The S4U2Proxy
+		 * evidence ticket could
+		 * not have been signed
+		 * or issued by a krbtgt
+		 * trust account.
+		 */
+		if (!krbtgt_skdc_entry->is_krbtgt) {
+			return EINVAL;
+		}
+		delegated_proxy_krbtgt_entry = krbtgt_skdc_entry;
 	}
 
+	delegated_proxy_pac_entry = samba_kdc_entry_pac(delegated_proxy_pac,
+							delegated_proxy_principal,
+							delegated_proxy_skdc_entry,
+							delegated_proxy_krbtgt_entry);
+
 	if (client != NULL) {
 		client_skdc_entry = talloc_get_type_abort(client->context,
 							  struct samba_kdc_entry);
 	}
 
-	if (device != NULL) {
-		const hdb_entry *device_krbtgt = NULL;
-
-		device_skdc_entry = talloc_get_type_abort(device->context,
-							  struct samba_kdc_entry);
-
-		device_krbtgt = kdc_request_get_explicit_armor_server(r);
-		device_krbtgt_skdc_entry = talloc_get_type_abort(device_krbtgt->context,
-								 struct samba_kdc_entry);
-	}
+	device = samba_kdc_get_device_pac(r);
 
 	ret = krb5_pac_init(context, &new_pac);
 	if (ret != 0) {
@@ -470,35 +349,27 @@ static krb5_error_code samba_wdc_reget_pac(void *priv, astgs_request_t r,
 		goto out;
 	}
 
-	if (krb5_pac_is_trusted(*pac)) {
-		flags |= SAMBA_KDC_FLAG_KRBTGT_IS_TRUSTED;
-	}
-	if (device_pac != NULL && krb5_pac_is_trusted(device_pac)) {
-		flags |= SAMBA_KDC_FLAG_DEVICE_KRBTGT_IS_TRUSTED;
-	}
-	if (delegated_proxy_pac != NULL && krb5_pac_is_trusted(delegated_proxy_pac)) {
-		flags |= SAMBA_KDC_FLAG_DELEGATED_PROXY_IS_TRUSTED;
+	client_pac_entry = samba_kdc_entry_pac(*pac,
+					       client_principal,
+					       client_skdc_entry,
+					       krbtgt_skdc_entry);
+
+	if (kdc_request_get_explicit_armor_present(r)) {
+		flags |= SAMBA_KDC_FLAG_EXPLICIT_ARMOR_PRESENT;
 	}
 
 	ret = samba_kdc_update_pac(mem_ctx,
 				   context,
-				   krbtgt_skdc_entry->kdc_db_ctx->samdb,
-				   krbtgt_skdc_entry->kdc_db_ctx->lp_ctx,
+				   krbtgt_skdc_entry->kdc_db_ctx,
 				   flags,
-				   client_krbtgt_skdc_entry,
-				   client_skdc_entry,
+				   client_pac_entry,
 				   server->principal,
 				   server_skdc_entry,
-				   delegated_proxy_principal,
-				   delegated_proxy_skdc_entry,
-				   delegated_proxy_pac,
-				   device_krbtgt_skdc_entry,
-				   device_skdc_entry,
-				   device_pac,
-				   *pac,
+				   delegated_proxy_pac_entry,
+				   device,
 				   new_pac,
 				   &server_audit_info,
-				   &status);
+				   &reply_status);
 	if (server_audit_info != NULL) {
 		krb5_error_code ret2;
 
@@ -507,10 +378,10 @@ static krb5_error_code samba_wdc_reget_pac(void *priv, astgs_request_t r,
 			ret = ret2;
 		}
 	}
-	if (!NT_STATUS_IS_OK(status)) {
+	if (!NT_STATUS_IS_OK(reply_status)) {
 		krb5_error_code ret2;
 
-		ret2 = hdb_samba4_set_ntstatus(r, status, ret);
+		ret2 = hdb_samba4_set_ntstatus(r, reply_status, ret);
 		if (ret == 0) {
 			ret = ret2;
 		}
@@ -537,14 +408,13 @@ out:
 /* Verify a PAC's SID and signatures */
 
 static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
-					    krb5_const_principal _client_principal,
+					    krb5_const_principal client_principal,
 					    hdb_entry *delegated_proxy,
 					    hdb_entry *client,
 					    hdb_entry *_server,
 					    hdb_entry *krbtgt,
 					    EncTicketPart *ticket,
-					    krb5_pac pac,
-					    krb5_boolean *is_trusted)
+					    krb5_pac pac)
 {
 	krb5_context context = kdc_request_get_context((kdc_request_t)r);
 	krb5_kdc_configuration *config = kdc_request_get_config((kdc_request_t)r);
@@ -554,14 +424,11 @@ static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
 	krb5_error_code ret;
 	krb5_cksumtype ctype = CKSUMTYPE_NONE;
 	hdb_entry signing_krbtgt_hdb;
-	const hdb_entry *explicit_armor_client =
-		kdc_request_get_explicit_armor_client(r);
-	krb5_const_pac explicit_armor_pac =
-		kdc_request_get_explicit_armor_pac(r);
 
 	if (delegated_proxy) {
-		uint16_t rodc_id;
-		unsigned int my_krbtgt_number;
+		uint16_t pac_kdc_signature_rodc_id;
+		const unsigned int local_tgs_rodc_id = krbtgt_skdc_entry->kdc_db_ctx->my_krbtgt_number;
+		const uint16_t header_ticket_rodc_id = krbtgt->kvno >> 16;
 
 		/*
 		 * We're using delegated_proxy for the moment to indicate cases
@@ -579,9 +446,9 @@ static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
 		ret = krb5_pac_get_kdc_checksum_info(context,
 						     pac,
 						     &ctype,
-						     &rodc_id);
+						     &pac_kdc_signature_rodc_id);
 		if (ret != 0) {
-			DEBUG(1, ("Failed to get PAC checksum info\n"));
+			DBG_WARNING("Failed to get PAC checksum info\n");
 			return ret;
 		}
 
@@ -589,16 +456,13 @@ static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
 		 * We need to check the KDC and ticket signatures, fetching the
 		 * correct key based on the enctype.
 		 */
-
-		my_krbtgt_number = krbtgt_skdc_entry->kdc_db_ctx->my_krbtgt_number;
-
-		if (my_krbtgt_number != 0) {
+		if (local_tgs_rodc_id != 0) {
 			/*
 			 * If we are an RODC, and we are not the KDC that signed
 			 * the evidence ticket, then we need to proxy the
 			 * request.
 			 */
-			if (rodc_id != my_krbtgt_number) {
+			if (local_tgs_rodc_id != pac_kdc_signature_rodc_id) {
 				return HDB_ERR_NOT_FOUND_HERE;
 			}
 		} else {
@@ -607,17 +471,9 @@ static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
 			 * different KDC than the one that issued the header
 			 * ticket.
 			 */
-			if (rodc_id != krbtgt->kvno >> 16) {
+			if (pac_kdc_signature_rodc_id != header_ticket_rodc_id) {
 				struct sdb_entry signing_krbtgt_sdb;
 
-				/*
-				 * If we didn't sign the ticket, then return an
-				 * error.
-				 */
-				if (rodc_id != 0) {
-					return KRB5KRB_AP_ERR_MODIFIED;
-				}
-
 				/*
 				 * Fetch our key from the database. To support
 				 * key rollover, we're going to need to try
@@ -627,8 +483,8 @@ static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
 				ret = samba_kdc_fetch(context,
 						      krbtgt_skdc_entry->kdc_db_ctx,
 						      krbtgt->principal,
-						      SDB_F_GET_KRBTGT | SDB_F_CANON,
-						      0,
+						      SDB_F_GET_KRBTGT | SDB_F_RODC_NUMBER_SPECIFIED | SDB_F_CANON,
+						      ((uint32_t)pac_kdc_signature_rodc_id) << 16,
 						      &signing_krbtgt_sdb);
 				if (ret != 0) {
 					return ret;
@@ -677,14 +533,12 @@ static krb5_error_code samba_wdc_verify_pac(void *priv, astgs_request_t r,
 	}
 
 	ret = samba_wdc_verify_pac2(r,
+				    client_principal,
 				    delegated_proxy,
 				    client,
 				    krbtgt,
 				    pac,
-				    ctype,
-				    explicit_armor_client,
-				    &explicit_armor_pac,
-				    is_trusted);
+				    ctype);
 
 	if (krbtgt == &signing_krbtgt_hdb) {
 		hdb_free_entry(context, config->db[0], &signing_krbtgt_hdb);
@@ -730,45 +584,44 @@ static krb5_error_code samba_wdc_check_client_access(void *priv,
 {
 	krb5_context context = kdc_request_get_context((kdc_request_t)r);
 	TALLOC_CTX *tmp_ctx = NULL;
-	const hdb_entry *client = NULL;
-	struct samba_kdc_entry *kdc_entry;
-	const hdb_entry *device = kdc_request_get_armor_client(r);
-	struct samba_kdc_entry *device_skdc_entry = NULL;
-	const krb5_const_pac device_pac = kdc_request_get_armor_pac(r);
+	const hdb_entry *_client = NULL;
+	struct samba_kdc_entry *client = NULL;
+	struct samba_kdc_db_context *kdc_db_ctx = NULL;
+	const hdb_entry *_server = NULL;
+	struct samba_kdc_entry *server = NULL;
+	struct samba_kdc_entry_pac device = {};
 	struct authn_audit_info *client_audit_info = NULL;
 	bool password_change;
 	char *workstation;
 	NTSTATUS nt_status;
 	NTSTATUS check_device_status = NT_STATUS_OK;
 	krb5_error_code ret = 0;
-	bool device_pac_is_trusted = false;
 
-	client = kdc_request_get_client(r);
+	/*
+	 * Note _kdc_as_rep() calls _kdc_check_access()
+	 * only after client and server are found
+	 * in the database!
+	 */
+	_client = kdc_request_get_client(r);
+	client = talloc_get_type_abort(_client->context, struct samba_kdc_entry);
+	kdc_db_ctx = client->kdc_db_ctx;
+	_server = kdc_request_get_server(r);
+	server = talloc_get_type_abort(_server->context, struct samba_kdc_entry);
+	/* We only have a single database! */
+	SMB_ASSERT(server->kdc_db_ctx == kdc_db_ctx);
 
-	tmp_ctx = talloc_named(client->context, 0, "samba_wdc_check_client_access");
+	tmp_ctx = talloc_named(client, 0, "samba_wdc_check_client_access");
 	if (tmp_ctx == NULL) {
 		return ENOMEM;
 	}
 
-	kdc_entry = talloc_get_type_abort(client->context, struct samba_kdc_entry);
-
-	if (device != NULL) {
-		device_skdc_entry = talloc_get_type_abort(device->context,
-							  struct samba_kdc_entry);
-	}
-
-	if (device_pac != NULL) {
-		device_pac_is_trusted = krb5_pac_is_trusted(device_pac);
-	}
+	device = samba_kdc_get_device_pac(r);
 
 	ret = samba_kdc_check_device(tmp_ctx,
 				     context,
-				     kdc_entry->kdc_db_ctx->samdb,
-				     kdc_entry->kdc_db_ctx->lp_ctx,
-				     device_skdc_entry,
-				     device_pac,
-				     device_pac_is_trusted,
-				     kdc_entry->client_policy,
+				     kdc_db_ctx,
+				     device,
+				     client->client_policy,
 				     &client_audit_info,
 				     &check_device_status);
 	if (client_audit_info != NULL) {
@@ -779,7 +632,7 @@ static krb5_error_code samba_wdc_check_client_access(void *priv,
 			ret = ret2;
 		}
 	}
-	kdc_entry->reject_status = check_device_status;
+	client->reject_status = check_device_status;
 	if (!NT_STATUS_IS_OK(check_device_status)) {
 		krb5_error_code ret2;
 
@@ -806,14 +659,14 @@ static krb5_error_code samba_wdc_check_client_access(void *priv,
 
 	workstation = get_netbios_name(tmp_ctx,
 				       kdc_request_get_req(r)->req_body.addresses);
-	password_change = (kdc_request_get_server(r) && kdc_request_get_server(r)->flags.change_pw);
+	password_change = _server->flags.change_pw;
 
-	nt_status = samba_kdc_check_client_access(kdc_entry,
+	nt_status = samba_kdc_check_client_access(client,
 						  kdc_request_get_cname((kdc_request_t)r),
 						  workstation,
 						  password_change);
 
-	kdc_entry->reject_status = nt_status;
+	client->reject_status = nt_status;
 	if (!NT_STATUS_IS_OK(nt_status)) {
 		krb5_error_code ret2;
 
@@ -884,12 +737,9 @@ static krb5_error_code samba_wdc_finalize_reply(void *priv,
 		md.padata_type = KRB5_PADATA_SUPPORTED_ETYPES;
 
 		ret = kdc_request_add_encrypted_padata(r, &md);
+		krb5_data_free(&md.padata_value);
 		if (ret != 0) {
-			/*
-			 * So we do not leak the allocated
-			 * memory on kd in the error case
-			 */
-			krb5_data_free(&md.padata_value);
+			return ret;
 		}
 	}
 
@@ -913,8 +763,27 @@ static krb5_error_code samba_wdc_referral_policy(void *priv,
 	return kdc_request_get_error_code((kdc_request_t)r);
 }
 
+static krb5_error_code samba_wdc_hwauth_policy(void *priv, astgs_request_t r)
+{
+	const hdb_entry *client = kdc_request_get_client(r);
+	krb5_error_code ret = 0;
+
+	if (client != NULL && client->flags.require_hwauth) {
+		krb5_error_code ret2;
+
+		ret = KRB5KDC_ERR_POLICY;
+		ret2 = hdb_samba4_set_ntstatus(
+			r, NT_STATUS_SMARTCARD_LOGON_REQUIRED, ret);
+		if (ret2) {
+			ret = ret2;
+		}
+	}
+
+	return ret;
+}
+
 struct krb5plugin_kdc_ftable kdc_plugin_table = {
-	.minor_version = KRB5_PLUGIN_KDC_VERSION_11,
+	.minor_version = KRB5_PLUGIN_KDC_VERSION_12,
 	.init = samba_wdc_plugin_init,
 	.fini = samba_wdc_plugin_fini,
 	.pac_verify = samba_wdc_verify_pac,
@@ -923,6 +792,5 @@ struct krb5plugin_kdc_ftable kdc_plugin_table = {
 	.finalize_reply = samba_wdc_finalize_reply,
 	.pac_generate = samba_wdc_get_pac,
 	.referral_policy = samba_wdc_referral_policy,
+	.hwauth_policy = samba_wdc_hwauth_policy,
 };
-
-
