@@ -1,19 +1,19 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    client file read/write routines
    Copyright (C) Andrew Tridgell 1994-1998
    Copyright (C) James Myers 2003
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -33,7 +33,7 @@
 _PUBLIC_ struct smbcli_request *smb_raw_read_send(struct smbcli_tree *tree, union smb_read *parms)
 {
 	bool bigoffset = false;
-	struct smbcli_request *req = NULL; 
+	struct smbcli_request *req = NULL;
 
 	switch (parms->generic.level) {
 	case RAW_READ_READBRAW:
@@ -104,12 +104,6 @@ _PUBLIC_ struct smbcli_request *smb_raw_read_send(struct smbcli_tree *tree, unio
 		return NULL;
 	}
 
-	/* the transport layer needs to know that a readbraw is pending
-	   and handle receives a little differently */
-	if (parms->generic.level == RAW_READ_READBRAW) {
-		tree->session->transport->readbraw_pending = 1;
-	}
-
 	return req;
 }
 
@@ -126,19 +120,19 @@ _PUBLIC_ NTSTATUS smb_raw_read_recv(struct smbcli_request *req, union smb_read *
 	switch (parms->generic.level) {
 	case RAW_READ_READBRAW:
 		parms->readbraw.out.nread = req->in.size - NBT_HDR_SIZE;
-		if (parms->readbraw.out.nread > 
+		if (parms->readbraw.out.nread >
 		    MAX(parms->readx.in.mincnt, parms->readx.in.maxcnt)) {
 			req->status = NT_STATUS_BUFFER_TOO_SMALL;
 			goto failed;
 		}
 		memcpy(parms->readbraw.out.data, req->in.buffer + NBT_HDR_SIZE, parms->readbraw.out.nread);
 		break;
-		
+
 	case RAW_READ_LOCKREAD:
 		SMBCLI_CHECK_WCT(req, 5);
 		parms->lockread.out.nread = SVAL(req->in.vwv, VWV(0));
 		if (parms->lockread.out.nread > parms->lockread.in.count ||
-		    !smbcli_raw_pull_data(&req->in.bufinfo, req->in.data+3, 
+		    !smbcli_raw_pull_data(&req->in.bufinfo, req->in.data+3,
 				       parms->lockread.out.nread, parms->lockread.out.data)) {
 			req->status = NT_STATUS_BUFFER_TOO_SMALL;
 		}
@@ -149,7 +143,7 @@ _PUBLIC_ NTSTATUS smb_raw_read_recv(struct smbcli_request *req, union smb_read *
 		SMBCLI_CHECK_WCT(req, 5);
 		parms->read.out.nread = SVAL(req->in.vwv, VWV(0));
 		if (parms->read.out.nread > parms->read.in.count ||
-		    !smbcli_raw_pull_data(&req->in.bufinfo, req->in.data+3, 
+		    !smbcli_raw_pull_data(&req->in.bufinfo, req->in.data+3,
 				       parms->read.out.nread, parms->read.out.data)) {
 			req->status = NT_STATUS_BUFFER_TOO_SMALL;
 		}
@@ -161,6 +155,8 @@ _PUBLIC_ NTSTATUS smb_raw_read_recv(struct smbcli_request *req, union smb_read *
 		parms->readx.out.remaining       = SVAL(req->in.vwv, VWV(2));
 		parms->readx.out.compaction_mode = SVAL(req->in.vwv, VWV(3));
 		parms->readx.out.nread = SVAL(req->in.vwv, VWV(5));
+		parms->readx.out.flags2 = req->flags2;
+		parms->readx.out.data_offset = SVAL(req->in.vwv, VWV(6));
 
 		/* handle oversize replies for non-chained readx replies with
 		   CAP_LARGE_READX. The snia spec has must to answer for. */
@@ -169,7 +165,7 @@ _PUBLIC_ NTSTATUS smb_raw_read_recv(struct smbcli_request *req, union smb_read *
 		    req->in.size >= 0x10000) {
 			parms->readx.out.nread += (SVAL(req->in.vwv, VWV(7)) << 16);
 			if (req->in.hdr + SVAL(req->in.vwv, VWV(6)) +
-			    parms->readx.out.nread <= 
+			    parms->readx.out.nread <=
 			    req->in.buffer + req->in.size) {
 				req->in.data_size += (SVAL(req->in.vwv, VWV(7)) << 16);
 
@@ -179,8 +175,8 @@ _PUBLIC_ NTSTATUS smb_raw_read_recv(struct smbcli_request *req, union smb_read *
 		}
 
 		if (parms->readx.out.nread > MAX(parms->readx.in.mincnt, parms->readx.in.maxcnt) ||
-		    !smbcli_raw_pull_data(&req->in.bufinfo, req->in.hdr + SVAL(req->in.vwv, VWV(6)), 
-				       parms->readx.out.nread, 
+		    !smbcli_raw_pull_data(&req->in.bufinfo, req->in.hdr + SVAL(req->in.vwv, VWV(6)),
+				       parms->readx.out.nread,
 				       parms->readx.out.data)) {
 			req->status = NT_STATUS_BUFFER_TOO_SMALL;
 		}
@@ -211,7 +207,7 @@ _PUBLIC_ NTSTATUS smb_raw_read(struct smbcli_tree *tree, union smb_read *parms)
 _PUBLIC_ struct smbcli_request *smb_raw_write_send(struct smbcli_tree *tree, union smb_write *parms)
 {
 	bool bigoffset = false;
-	struct smbcli_request *req = NULL; 
+	struct smbcli_request *req = NULL;
 
 	switch (parms->generic.level) {
 	case RAW_WRITE_WRITEUNLOCK:
@@ -223,7 +219,7 @@ _PUBLIC_ struct smbcli_request *smb_raw_write_send(struct smbcli_tree *tree, uni
 		SCVAL(req->out.data, 0, SMB_DATA_BLOCK);
 		SSVAL(req->out.data, 1, parms->writeunlock.in.count);
 		if (parms->writeunlock.in.count > 0) {
-			memcpy(req->out.data+3, parms->writeunlock.in.data, 
+			memcpy(req->out.data+3, parms->writeunlock.in.data,
 			       parms->writeunlock.in.count);
 		}
 		break;
@@ -250,7 +246,7 @@ _PUBLIC_ struct smbcli_request *smb_raw_write_send(struct smbcli_tree *tree, uni
 				  req->out.vwv, VWV(4), parms->writeclose.in.mtime);
 		SCVAL(req->out.data, 0, 0);
 		if (parms->writeclose.in.count > 0) {
-			memcpy(req->out.data+1, parms->writeclose.in.data, 
+			memcpy(req->out.data+1, parms->writeclose.in.data,
 			       parms->writeclose.in.count);
 		}
 		break;
@@ -302,7 +298,7 @@ _PUBLIC_ struct smbcli_request *smb_raw_write_send(struct smbcli_tree *tree, uni
 /****************************************************************************
  raw write interface (async recv)
 ****************************************************************************/
-NTSTATUS smb_raw_write_recv(struct smbcli_request *req, union smb_write *parms)
+_PUBLIC_ NTSTATUS smb_raw_write_recv(struct smbcli_request *req, union smb_write *parms)
 {
 	if (!smbcli_request_receive(req) ||
 	    !NT_STATUS_IS_OK(req->status)) {
@@ -311,7 +307,7 @@ NTSTATUS smb_raw_write_recv(struct smbcli_request *req, union smb_write *parms)
 
 	switch (parms->generic.level) {
 	case RAW_WRITE_WRITEUNLOCK:
-		SMBCLI_CHECK_WCT(req, 1);		
+		SMBCLI_CHECK_WCT(req, 1);
 		parms->writeunlock.out.nwritten = SVAL(req->in.vwv, VWV(0));
 		break;
 	case RAW_WRITE_WRITE:
