@@ -1,20 +1,20 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    SMB Signing Code
    Copyright (C) Jeremy Allison 2002.
    Copyright (C) Andrew Bartlett <abartlet@samba.org> 2002-2003
    Copyright (C) James J Myers <myersjj@samba.org> 2003
-   
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
@@ -24,6 +24,9 @@
 #include "libcli/raw/raw_proto.h"
 #include "../lib/crypto/crypto.h"
 
+#include <gnutls/gnutls.h>
+#include <gnutls/crypto.h>
+
 /***********************************************************
  SMB signing - Common code before we set a new signing implementation
 ************************************************************/
@@ -42,29 +45,7 @@ bool set_smb_signing_common(struct smb_signing_context *sign_info)
 	return true;
 }
 
-/***********************************************************
- SMB signing - Common code before we set a new signing implementation
-************************************************************/
-static bool smbcli_set_smb_signing_common(struct smbcli_transport *transport)
-{
-	if (!set_smb_signing_common(&transport->negotiate.sign_info)) {
-		return false;
-	}
-
-	if (!(transport->negotiate.sec_mode & 
-	      (NEGOTIATE_SECURITY_SIGNATURES_REQUIRED|NEGOTIATE_SECURITY_SIGNATURES_ENABLED))) {
-		DEBUG(5, ("SMB Signing is not negotiated by the peer\n"));
-		return false;
-	}
-
-	/* These calls are INCOMPATIBLE with SMB signing */
-	transport->negotiate.readbraw_supported = false;
-	transport->negotiate.writebraw_supported = false;
-
-	return true;
-}
-
-void mark_packet_signed(struct smb_request_buffer *out) 
+void mark_packet_signed(struct smb_request_buffer *out)
 {
 	uint16_t flags2;
 	flags2 = SVAL(out->hdr, HDR_FLG2);
@@ -72,8 +53,8 @@ void mark_packet_signed(struct smb_request_buffer *out)
 	SSVAL(out->hdr, HDR_FLG2, flags2);
 }
 
-bool signing_good(struct smb_signing_context *sign_info, 
-			 unsigned int seq, bool good) 
+bool signing_good(struct smb_signing_context *sign_info,
+			 unsigned int seq, bool good)
 {
 	if (good) {
 		if (!sign_info->doing_signing) {
@@ -100,10 +81,11 @@ bool signing_good(struct smb_signing_context *sign_info,
 	return true;
 }
 
-void sign_outgoing_message(struct smb_request_buffer *out, DATA_BLOB *mac_key, unsigned int seq_num) 
+void sign_outgoing_message(struct smb_request_buffer *out, DATA_BLOB *mac_key, unsigned int seq_num)
 {
 	uint8_t calc_md5_mac[16];
-	MD5_CTX md5_ctx;
+	gnutls_hash_hd_t hash_hnd = NULL;
+	int rc;
 
 	/*
 	 * Firstly put the sequence number into the first 4 bytes.
@@ -116,30 +98,44 @@ void sign_outgoing_message(struct smb_request_buffer *out, DATA_BLOB *mac_key, u
 	mark_packet_signed(out);
 
 	/* Calculate the 16 byte MAC and place first 8 bytes into the field. */
-	MD5Init(&md5_ctx);
-        MD5Update(&md5_ctx, mac_key->data, mac_key->length);
-	MD5Update(&md5_ctx, 
-		  out->buffer + NBT_HDR_SIZE, 
-		  out->size - NBT_HDR_SIZE);
-	MD5Final(calc_md5_mac, &md5_ctx);
+	rc = gnutls_hash_init(&hash_hnd, GNUTLS_DIG_MD5);
+	if (rc < 0) {
+		return;
+	}
+
+	rc = gnutls_hash(hash_hnd, mac_key->data, mac_key->length);
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return;
+	}
+	rc = gnutls_hash(hash_hnd,
+			 out->buffer + NBT_HDR_SIZE,
+			 out->size - NBT_HDR_SIZE);
+	if (rc < 0) {
+		gnutls_hash_deinit(hash_hnd, NULL);
+		return;
+	}
+	gnutls_hash_deinit(hash_hnd, calc_md5_mac);
 
 	memcpy(&out->hdr[HDR_SS_FIELD], calc_md5_mac, 8);
 
-	DEBUG(5, ("sign_outgoing_message: SENT SIG (seq: %d): sent SMB signature of\n", 
+	DEBUG(5, ("sign_outgoing_message: SENT SIG (seq: %d): sent SMB signature of\n",
 		  seq_num));
 	dump_data(5, calc_md5_mac, 8);
-/*	req->out.hdr[HDR_SS_FIELD+2]=0; 
+	ZERO_ARRAY(calc_md5_mac);
+/*	req->out.hdr[HDR_SS_FIELD+2]=0;
 	Uncomment this to test if the remote server actually verifies signitures...*/
 }
 
 bool check_signed_incoming_message(struct smb_request_buffer *in, DATA_BLOB *mac_key, unsigned int seq_num)
 {
-	bool good;
+	bool ok = false;
 	uint8_t calc_md5_mac[16];
 	uint8_t *server_sent_mac;
 	uint8_t sequence_buf[8];
-	MD5_CTX md5_ctx;
+	gnutls_hash_hd_t hash_hnd;
 	const size_t offset_end_of_sig = (HDR_SS_FIELD + 8);
+	int rc;
 	int i;
 	const int sign_range = 0;
 
@@ -162,28 +158,53 @@ bool check_signed_incoming_message(struct smb_request_buffer *in, DATA_BLOB *mac
 		 */
 		SIVAL(sequence_buf, 0, seq_num + i);
 		SIVAL(sequence_buf, 4, 0);
-		
+
 		/* get a copy of the server-sent mac */
 	        server_sent_mac = &in->hdr[HDR_SS_FIELD];
-		
+
 		/* Calculate the 16 byte MAC and place first 8 bytes into the field. */
-		MD5Init(&md5_ctx);
-		MD5Update(&md5_ctx, mac_key->data, 
-			  mac_key->length); 
-		MD5Update(&md5_ctx, in->hdr, HDR_SS_FIELD);
-		MD5Update(&md5_ctx, sequence_buf, sizeof(sequence_buf));
-		
-		MD5Update(&md5_ctx, in->hdr + offset_end_of_sig, 
-			  in->size - NBT_HDR_SIZE - (offset_end_of_sig));
-		MD5Final(calc_md5_mac, &md5_ctx);
-		
-		good = (memcmp(server_sent_mac, calc_md5_mac, 8) == 0);
+		rc = gnutls_hash_init(&hash_hnd, GNUTLS_DIG_MD5);
+		if (rc < 0) {
+			ok = false;
+			goto out;
+		}
+
+		rc = gnutls_hash(hash_hnd, mac_key->data, mac_key->length);
+		if (rc < 0) {
+			gnutls_hash_deinit(hash_hnd, NULL);
+			ok = false;
+			goto out;
+		}
+		rc = gnutls_hash(hash_hnd, in->hdr, HDR_SS_FIELD);
+		if (rc < 0) {
+			gnutls_hash_deinit(hash_hnd, NULL);
+			ok = false;
+			goto out;
+		}
+		rc = gnutls_hash(hash_hnd, sequence_buf, sizeof(sequence_buf));
+		if (rc < 0) {
+			gnutls_hash_deinit(hash_hnd, NULL);
+			ok = false;
+			goto out;
+		}
+		rc = gnutls_hash(hash_hnd,
+				 in->hdr + offset_end_of_sig,
+				 in->size - NBT_HDR_SIZE - (offset_end_of_sig));
+		if (rc < 0) {
+			gnutls_hash_deinit(hash_hnd, NULL);
+			ok = false;
+			goto out;
+		}
+
+		gnutls_hash_deinit(hash_hnd, calc_md5_mac);
+
+		ok = mem_equal_const_time(server_sent_mac, calc_md5_mac, 8);
 
 		if (i == 0) {
-			if (!good) {
+			if (!ok) {
 				DEBUG(5, ("check_signed_incoming_message: BAD SIG (seq: %d): wanted SMB signature of\n", seq_num + i));
 				dump_data(5, calc_md5_mac, 8);
-				
+
 				DEBUG(5, ("check_signed_incoming_message: BAD SIG (seq: %d): got SMB signature of\n", seq_num + i));
 				dump_data(5, server_sent_mac, 8);
 			} else {
@@ -191,66 +212,19 @@ bool check_signed_incoming_message(struct smb_request_buffer *in, DATA_BLOB *mac
 				dump_data(5, server_sent_mac, 8);
 			}
 		}
+		ZERO_ARRAY(calc_md5_mac);
 
-		if (good) break;
+		if (ok) break;
 	}
 
-	if (good && i != 0) {
+	if (ok && i != 0) {
 		DEBUG(0,("SIGNING OFFSET %d (should be %d)\n", i, seq_num));
 	}
 
-	return good;
+out:
+	return ok;
 }
 
-static void smbcli_req_allocate_seq_num(struct smbcli_request *req) 
-{
-	req->seq_num = req->transport->negotiate.sign_info.next_seq_num;
-	
-	/* some requests (eg. NTcancel) are one way, and the sequence number
-	   should be increased by 1 not 2 */
-	if (req->sign_single_increment) {
-		req->transport->negotiate.sign_info.next_seq_num += 1;
-	} else {
-		req->transport->negotiate.sign_info.next_seq_num += 2;
-	}
-}
-
-/***********************************************************
- SMB signing - Simple implementation - calculate a MAC to send.
-************************************************************/
-void smbcli_request_calculate_sign_mac(struct smbcli_request *req)
-{
-#if 0
-	/* enable this when packet signing is preventing you working out why valgrind 
-	   says that data is uninitialised */
-	file_save("pkt.dat", req->out.buffer, req->out.size);
-#endif
-
-	switch (req->transport->negotiate.sign_info.signing_state) {
-	case SMB_SIGNING_ENGINE_OFF:
-		break;
-
-	case SMB_SIGNING_ENGINE_BSRSPYL:
-		/* mark the packet as signed - BEFORE we sign it...*/
-		mark_packet_signed(&req->out);
-		
-		/* I wonder what BSRSPYL stands for - but this is what MS 
-		   actually sends! */
-		memcpy((req->out.hdr + HDR_SS_FIELD), "BSRSPYL ", 8);
-		break;
-
-	case SMB_SIGNING_ENGINE_ON:
-			
-		smbcli_req_allocate_seq_num(req);
-		sign_outgoing_message(&req->out, 
-				      &req->transport->negotiate.sign_info.mac_key, 
-				      req->seq_num);
-		break;
-	}
-	return;
-}
-
-
 /**
  SMB signing - NULL implementation
 
@@ -266,74 +240,12 @@ bool smbcli_set_signing_off(struct smb_signing_context *sign_info)
 	return true;
 }
 
-/**
- SMB signing - TEMP implementation - setup the MAC key.
-
-*/
-bool smbcli_temp_set_signing(struct smbcli_transport *transport)
-{
-	if (!smbcli_set_smb_signing_common(transport)) {
-		return false;
-	}
-	DEBUG(5, ("BSRSPYL SMB signing enabled\n"));
-	smbcli_set_signing_off(&transport->negotiate.sign_info);
-
-	transport->negotiate.sign_info.mac_key = data_blob(NULL, 0);
-	transport->negotiate.sign_info.signing_state = SMB_SIGNING_ENGINE_BSRSPYL;
-
-	return true;
-}
-
-/***********************************************************
- SMB signing - Simple implementation - check a MAC sent by server.
-************************************************************/
-/**
- * Check a packet supplied by the server.
- * @return false if we had an established signing connection
- *         which had a back checksum, true otherwise
- */
-bool smbcli_request_check_sign_mac(struct smbcli_request *req) 
-{
-	bool good;
-
-	if (!req->transport->negotiate.sign_info.doing_signing &&
-	    req->sign_caller_checks) {
-		return true;
-	}
-
-	req->sign_caller_checks = false;
-
-	switch (req->transport->negotiate.sign_info.signing_state) 
-	{
-	case SMB_SIGNING_ENGINE_OFF:
-		return true;
-	case SMB_SIGNING_ENGINE_BSRSPYL:
-		return true;
-
-	case SMB_SIGNING_ENGINE_ON:
-	{			
-		if (req->in.size < (HDR_SS_FIELD + 8)) {
-			return false;
-		} else {
-			good = check_signed_incoming_message(&req->in, 
-							     &req->transport->negotiate.sign_info.mac_key, 
-							     req->seq_num+1);
-			
-			return signing_good(&req->transport->negotiate.sign_info, 
-					    req->seq_num+1, good);
-		}
-	}
-	}
-	return false;
-}
-
-
 /***********************************************************
  SMB signing - Simple implementation - setup the MAC key.
 ************************************************************/
 bool smbcli_simple_set_signing(TALLOC_CTX *mem_ctx,
 			       struct smb_signing_context *sign_info,
-			       const DATA_BLOB *user_session_key, 
+			       const DATA_BLOB *user_session_key,
 			       const DATA_BLOB *response)
 {
 	if (sign_info->mandatory_signing) {
@@ -347,7 +259,7 @@ bool smbcli_simple_set_signing(TALLOC_CTX *mem_ctx,
 	} else {
 		sign_info->mac_key = data_blob_talloc(mem_ctx, NULL, user_session_key->length);
 	}
-		
+
 	memcpy(&sign_info->mac_key.data[0], user_session_key->data, user_session_key->length);
 
 	if (response && response->length) {
@@ -361,46 +273,3 @@ bool smbcli_simple_set_signing(TALLOC_CTX *mem_ctx,
 
 	return true;
 }
-
-
-/***********************************************************
- SMB signing - Simple implementation - setup the MAC key.
-************************************************************/
-bool smbcli_transport_simple_set_signing(struct smbcli_transport *transport,
-					 const DATA_BLOB user_session_key, 
-					 const DATA_BLOB response)
-{
-	if (!smbcli_set_smb_signing_common(transport)) {
-		return false;
-	}
-
-	return smbcli_simple_set_signing(transport,
-					 &transport->negotiate.sign_info,
-					 &user_session_key,
-					 &response);
-}
-
-
-bool smbcli_init_signing(struct smbcli_transport *transport) 
-{
-	transport->negotiate.sign_info.next_seq_num = 0;
-	transport->negotiate.sign_info.mac_key = data_blob(NULL, 0);
-	if (!smbcli_set_signing_off(&transport->negotiate.sign_info)) {
-		return false;
-	}
-	
-	switch (transport->options.signing) {
-	case SMB_SIGNING_OFF:
-		transport->negotiate.sign_info.allow_smb_signing = false;
-		break;
-	case SMB_SIGNING_SUPPORTED:
-	case SMB_SIGNING_AUTO:
-		transport->negotiate.sign_info.allow_smb_signing = true;
-		break;
-	case SMB_SIGNING_REQUIRED:
-		transport->negotiate.sign_info.allow_smb_signing = true;
-		transport->negotiate.sign_info.mandatory_signing = true;
-		break;
-	}
-	return true;
-}
