@@ -88,7 +88,7 @@ static NTSTATUS fetchfile_read(struct composite_context *c,
 static void fetchfile_state_handler(struct composite_context *c)
 {
 	struct fetchfile_state *state;
-	NTSTATUS status = NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS status;
 	
 	state = talloc_get_type(c->private_data, struct fetchfile_state);
 
@@ -101,6 +101,9 @@ static void fetchfile_state_handler(struct composite_context *c)
 	case FETCHFILE_READ:
 		status = fetchfile_read(c, state->io);
 		break;
+	default:
+		status = NT_STATUS_UNSUCCESSFUL;
+		break;
 	}
 
 	if (!NT_STATUS_IS_OK(status)) {
@@ -120,6 +123,7 @@ static void fetchfile_composite_handler(struct composite_context *creq)
 }
 
 struct composite_context *smb_composite_fetchfile_send(struct smb_composite_fetchfile *io,
+						       struct loadparm_context *lp_ctx,
 						       struct tevent_context *event_ctx)
 {
 	struct composite_context *c;
@@ -131,13 +135,12 @@ struct composite_context *smb_composite_fetchfile_send(struct smb_composite_fetc
 	state = talloc(c, struct fetchfile_state);
 	if (state == NULL) goto failed;
 
-	state->connect = talloc(state, struct smb_composite_connect);
+	state->connect = talloc_zero(state, struct smb_composite_connect);
 	if (state->connect == NULL) goto failed;
 
 	state->io = io;
 
 	state->connect->in.dest_host    = io->in.dest_host;
-	state->connect->in.dest_ports   = io->in.ports;
 	state->connect->in.socket_options = io->in.socket_options;
 	state->connect->in.called_name  = io->in.called_name;
 	state->connect->in.service      = io->in.service;
@@ -146,12 +149,11 @@ struct composite_context *smb_composite_fetchfile_send(struct smb_composite_fetc
 	state->connect->in.fallback_to_anonymous = false;
 	state->connect->in.workgroup    = io->in.workgroup;
 	state->connect->in.gensec_settings = io->in.gensec_settings;
-	state->connect->in.iconv_convenience = io->in.iconv_convenience;
 
 	state->connect->in.options	= io->in.options;
 	state->connect->in.session_options = io->in.session_options;
 
-	state->creq = smb_composite_connect_send(state->connect, state, 
+	state->creq = smb_composite_connect_send(state->connect, state, lp_ctx,
 						 io->in.resolve_ctx, event_ctx);
 	if (state->creq == NULL) goto failed;
 
@@ -183,10 +185,3 @@ NTSTATUS smb_composite_fetchfile_recv(struct composite_context *c,
 	talloc_free(c);
 	return status;
 }
-
-NTSTATUS smb_composite_fetchfile(struct smb_composite_fetchfile *io,
-				 TALLOC_CTX *mem_ctx)
-{
-	struct composite_context *c = smb_composite_fetchfile_send(io, NULL);
-	return smb_composite_fetchfile_recv(c, mem_ctx);
-}
