@@ -29,40 +29,29 @@
 
 enum handle_types { HTYPE_REGVAL, HTYPE_REGKEY };
 
-static NTSTATUS dcerpc_winreg_bind(struct dcesrv_call_state *dce_call,
-				   const struct dcesrv_interface *iface)
-{
-	struct registry_context *ctx;
-	WERROR err;
-
-	err = reg_open_samba(dce_call->context,
-			     &ctx, dce_call->event_ctx, dce_call->conn->dce_ctx->lp_ctx,
-			     dce_call->conn->auth_state.session_info,
-			     NULL);
-
-	if (!W_ERROR_IS_OK(err)) {
-		DEBUG(0, ("Error opening registry: %s\n", win_errstr(err)));
-		return werror_to_ntstatus(err);
-	}
-
-	dce_call->context->private_data = ctx;
-
-	return NT_STATUS_OK;
-}
-
-#define DCESRV_INTERFACE_WINREG_BIND dcerpc_winreg_bind
-
 static WERROR dcesrv_winreg_openhive(struct dcesrv_call_state *dce_call,
 				     TALLOC_CTX *mem_ctx, uint32_t hkey,
 				     struct policy_handle **outh)
 {
-	struct registry_context *ctx = dce_call->context->private_data;
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
+	struct registry_context *ctx = NULL;
 	struct dcesrv_handle *h;
 	WERROR result;
 
-	h = dcesrv_handle_new(dce_call->context, HTYPE_REGKEY);
+	h = dcesrv_handle_create(dce_call, HTYPE_REGKEY);
 	W_ERROR_HAVE_NO_MEMORY(h);
 
+	result = reg_open_samba(h, &ctx,
+				dce_call->event_ctx,
+				dce_call->conn->dce_ctx->lp_ctx,
+				session_info,
+				NULL);
+	if (!W_ERROR_IS_OK(result)) {
+		DEBUG(0, ("Error opening registry: %s\n", win_errstr(result)));
+		return result;
+	}
+
 	result = reg_get_predefined_key(ctx, hkey,
 				       (struct registry_key **)&h->data);
 	if (!W_ERROR_IS_OK(result)) {
@@ -113,6 +102,8 @@ static WERROR dcesrv_winreg_CreateKey(struct dcesrv_call_state *dce_call,
 				      TALLOC_CTX *mem_ctx,
 				      struct winreg_CreateKey *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h, *newh;
 	struct security_descriptor sd;
 	struct registry_key *key;
@@ -121,9 +112,9 @@ static WERROR dcesrv_winreg_CreateKey(struct dcesrv_call_state *dce_call,
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	newh = dcesrv_handle_new(dce_call->context, HTYPE_REGKEY);
+	newh = dcesrv_handle_create(dce_call, HTYPE_REGKEY);
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
@@ -139,12 +130,12 @@ static WERROR dcesrv_winreg_CreateKey(struct dcesrv_call_state *dce_call,
 			sdblob.data = r->in.secdesc->sd.data;
 			sdblob.length = r->in.secdesc->sd.len;
 			if (sdblob.data == NULL) {
-				return WERR_INVALID_PARAM;
+				return WERR_INVALID_PARAMETER;
 			}
 			ndr_err = ndr_pull_struct_blob_all(&sdblob, mem_ctx, &sd,
 							   (ndr_pull_flags_fn_t)ndr_pull_security_descriptor);
 			if (!NDR_ERR_CODE_IS_SUCCESS(ndr_err)) {
-				return WERR_INVALID_PARAM;
+				return WERR_INVALID_PARAMETER;
 			}
 		}
 		
@@ -154,7 +145,7 @@ static WERROR dcesrv_winreg_CreateKey(struct dcesrv_call_state *dce_call,
 		r->out.action_taken = talloc(mem_ctx, enum winreg_CreateAction);
 		if (r->out.action_taken == NULL) {
 			talloc_free(newh);
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 		*r->out.action_taken = REG_ACTION_NONE;
 
@@ -179,6 +170,8 @@ static WERROR dcesrv_winreg_DeleteKey(struct dcesrv_call_state *dce_call,
 				      TALLOC_CTX *mem_ctx,
 				      struct winreg_DeleteKey *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h;
 	struct registry_key *key;
 	WERROR result;
@@ -186,7 +179,7 @@ static WERROR dcesrv_winreg_DeleteKey(struct dcesrv_call_state *dce_call,
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
@@ -207,13 +200,15 @@ static WERROR dcesrv_winreg_DeleteValue(struct dcesrv_call_state *dce_call,
 					TALLOC_CTX *mem_ctx,
 					struct winreg_DeleteValue *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h;
 	struct registry_key *key;
 
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
@@ -313,10 +308,14 @@ static WERROR dcesrv_winreg_EnumValue(struct dcesrv_call_state *dce_call,
 
 	r->out.type = talloc(mem_ctx, enum winreg_Type);
 	if (!r->out.type) {
-		return WERR_NOMEM;
+		return WERR_NOT_ENOUGH_MEMORY;
 	}
 	*r->out.type = (enum winreg_Type) data_type;
 
+	if (r->in.size != NULL) {
+		r->out.size = talloc(mem_ctx, uint32_t);
+		*r->out.size = data.length;
+	}
 	/* check the client has enough room for the value */
 	if (r->in.value != NULL &&
 	    r->in.size != NULL &&
@@ -328,9 +327,7 @@ static WERROR dcesrv_winreg_EnumValue(struct dcesrv_call_state *dce_call,
 		r->out.value = data.data;
 	}
 
-	if (r->in.size != NULL) {
-		r->out.size = talloc(mem_ctx, uint32_t);
-		*r->out.size = data.length;
+	if (r->in.length != NULL) {
 		r->out.length = r->out.size;
 	}
 
@@ -345,13 +342,15 @@ static WERROR dcesrv_winreg_FlushKey(struct dcesrv_call_state *dce_call,
 				     TALLOC_CTX *mem_ctx,
 				     struct winreg_FlushKey *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h;
 	struct registry_key *key;
 
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
@@ -406,6 +405,8 @@ static WERROR dcesrv_winreg_OpenKey(struct dcesrv_call_state *dce_call,
 				    TALLOC_CTX *mem_ctx,
 				    struct winreg_OpenKey *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h, *newh;
 	struct registry_key *key;
 	WERROR result;
@@ -413,7 +414,7 @@ static WERROR dcesrv_winreg_OpenKey(struct dcesrv_call_state *dce_call,
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.parent_handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
@@ -422,7 +423,7 @@ static WERROR dcesrv_winreg_OpenKey(struct dcesrv_call_state *dce_call,
 			newh = talloc_reference(dce_call->context, h);
 			result = WERR_OK;
 		} else {
-			newh = dcesrv_handle_new(dce_call->context, HTYPE_REGKEY);
+			newh = dcesrv_handle_create(dce_call, HTYPE_REGKEY);
 			result = reg_open_key(newh, key, r->in.keyname.name,
 				(struct registry_key **)&newh->data);
 		}
@@ -446,6 +447,8 @@ static WERROR dcesrv_winreg_QueryInfoKey(struct dcesrv_call_state *dce_call,
 					 TALLOC_CTX *mem_ctx,
 					 struct winreg_QueryInfoKey *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h;
 	struct registry_key *key;
 	const char *classname = NULL;
@@ -454,7 +457,7 @@ static WERROR dcesrv_winreg_QueryInfoKey(struct dcesrv_call_state *dce_call,
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
@@ -496,6 +499,8 @@ static WERROR dcesrv_winreg_QueryValue(struct dcesrv_call_state *dce_call,
 				       TALLOC_CTX *mem_ctx,
 				       struct winreg_QueryValue *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h;
 	struct registry_key *key;
 	uint32_t value_type;
@@ -505,14 +510,14 @@ static WERROR dcesrv_winreg_QueryValue(struct dcesrv_call_state *dce_call,
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
 	case SECURITY_USER:
 		if ((r->in.type == NULL) || (r->in.data_length == NULL) ||
 		    (r->in.data_size == NULL)) {
-			return WERR_INVALID_PARAM;
+			return WERR_INVALID_PARAMETER;
 		}
 
 		result = reg_key_get_value_by_name(mem_ctx, key, 
@@ -532,17 +537,17 @@ static WERROR dcesrv_winreg_QueryValue(struct dcesrv_call_state *dce_call,
 
 		r->out.type = talloc(mem_ctx, enum winreg_Type);
 		if (!r->out.type) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 		*r->out.type = (enum winreg_Type) value_type;
 		r->out.data_length = talloc(mem_ctx, uint32_t);
 		if (!r->out.data_length) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 		*r->out.data_length = value_data.length;
 		r->out.data_size = talloc(mem_ctx, uint32_t);
 		if (!r->out.data_size) {
-			return WERR_NOMEM;
+			return WERR_NOT_ENOUGH_MEMORY;
 		}
 		*r->out.data_size = value_data.length;
 		r->out.data = value_data.data;
@@ -605,6 +610,8 @@ static WERROR dcesrv_winreg_SetValue(struct dcesrv_call_state *dce_call,
 				     TALLOC_CTX *mem_ctx,
 				     struct winreg_SetValue *r)
 {
+	struct auth_session_info *session_info =
+		dcesrv_call_session_info(dce_call);
 	struct dcesrv_handle *h;
 	struct registry_key *key;
 	DATA_BLOB data;
@@ -613,7 +620,7 @@ static WERROR dcesrv_winreg_SetValue(struct dcesrv_call_state *dce_call,
 	DCESRV_PULL_HANDLE_FAULT(h, r->in.handle, HTYPE_REGKEY);
 	key = h->data;
 
-	switch (security_session_user_level(dce_call->conn->auth_state.session_info, NULL))
+	switch (security_session_user_level(session_info, NULL))
 	{
 	case SECURITY_SYSTEM:
 	case SECURITY_ADMINISTRATOR:
